/***********************************************************************
*
*                           Xg3dplt.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.2   11/92
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "math.h"
#include "time.h"
#include "X11/Intrinsic.h"
#include "X11/Xlib.h"
#include "X11/Xutil.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/Form.h"
#include "Xm/DrawingA.h"

#include "Xg3d.h"
#define DEV_COORD Dimension

/** comment out next line to disable row plot ability **/
/*#define ENABLE_ROW_PLOT*/
#ifdef ENABLE_ROW_PLOT
#include "XgPlot.h"
#endif

#define HOME    65360
#define LEFTA   65361
#define UPA     65362
#define RIGHTA  65363
#define DOWNA   65364
#define MK_ROW    103
/* g=103, G=71, a=97, A=65, t=116, T=84. Ascii, get it? */

void Xg3dDrawPoint(double, double, Xg3dGraphInfo *);
void Xg3dDrawCircle(double, double, int, Xg3dGraphInfo *);
void Xg3dDrawSquare(double, double, int, Xg3dGraphInfo *);
void Xg3dDrawBar(double, double, Xg3dGraphInfo *);
void Xg3dDrawTriangle(double, double, int, Xg3dGraphInfo *);
void Xg3dDrawLine(double, double, double, double, Xg3dGraphInfo *);
GC Xg3dCreateGC(Xg3dGraphInfo *);
void Xg3dSetworld(double, double, double, double, Xg3dGraphInfo *);
void Xg3dMapwtod(double, double, DEV_COORD *, DEV_COORD *, Xg3dGraphInfo *);
void Xg3dMapdtow(int, int, double *, double *, Xg3dGraphInfo *);
double Xg3dScale_Func(int, int, Xg3dGraphInfo *);
void Xg3d_redrawExpose();
void Xg3d_Print();
void Xg3d_processArrow();

/****************************************/
Xg3d_createGraph(graphinfo)
     Xg3dGraphInfo
       *graphinfo;
{
  int
    i,
    j,
    nx,
    ny,
    x,
    y,
    xmin,
    xmax,
    ymin,
    ymax;
  double
    inc,
    dx,
    hoff,
    scale_fac;
  char
    buf[20],
    cnum[20],
    *chrptr;
/*************** set min, max values *************/
graphinfo->yaxis_layout.win_min = 65536;
graphinfo->yaxis_layout.win_max = -65536;
dx = graphinfo->hoff * tan(graphinfo->theta * 3.14159 / 180.);
hoff = graphinfo->hoff;
scale_fac = graphinfo->scale_fac;
nx = graphinfo->dataset[0].nx;
ny = graphinfo->dataset[0].ny;
xmin = (int) graphinfo->xaxis_layout.min;
xmax = (int) graphinfo->xaxis_layout.max;
ymin = (int) graphinfo->yaxis_layout.min;
ymax = (int) graphinfo->yaxis_layout.max;
graphinfo->xaxis_layout.win_min = xmin;
graphinfo->xaxis_layout.win_max = xmax + (int) (dx*(ymax - ymin));
for (y=ymin; y<ymax; ++y)
  {
    for (x=xmin; x<xmax; ++x)
      {
	if (( (y-ymin) * hoff + Xg3dScale_Func(x, y, graphinfo) * scale_fac)
	    < graphinfo->yaxis_layout.win_min)
	  graphinfo->yaxis_layout.win_min = (y - ymin) * hoff
	    + Xg3dScale_Func(x, y, graphinfo) * scale_fac;
	if (( (y-ymin) * hoff + Xg3dScale_Func(x, y, graphinfo) * scale_fac)
	    > graphinfo->yaxis_layout.win_max)
	  graphinfo->yaxis_layout.win_max = (y - ymin) * hoff
	    + Xg3dScale_Func(x, y, graphinfo) * scale_fac;
      }
  }

#ifdef JUNK
/************** adjust to labeling parameters ************/
/** We do this adjustment regardless of Xg3dUmax/min flag **/

inc = graphinfo->xaxis_layout.increment - Xg3dLast0;
sprintf(buf, "%le", graphinfo->xaxis_layout.min);  /* get sci notation form */
sscanf(buf, "%[^e]", cnum);
sscanf(cnum, "%le", &xmin);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%le", &ymin);
sprintf(buf, "%le", graphinfo->xaxis_layout.max);
sscanf(buf, "%[^e]", cnum);
sscanf(cnum, "%le", &xmax);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%le", &ymax);
if (graphinfo->xaxis_layout.nsig_fig == Xg3dSig2) /* scale to proper sig. digit*/
  {
    xmin = xmin * 10;
    xmax = xmax * 10;
  }
if (graphinfo->xaxis_layout.nsig_fig == Xg3dSig3)
  {
    xmin = xmin * 100;
    xmax = xmax * 100;
  }
if ((xmax != 0) && (xmin != 0))
  {
    if (ymax > ymin)          /* adjust both to same scale */
      xmin = xmin / pow(10., ymax - ymin);
    else if (ymax < ymin)
      xmax = xmax / pow(10., ymin - ymax);
  }
xmin = floor(xmin * 1.00001 / inc) * inc; /* move up or down to next level */
xmax = ceil(xmax / inc) * inc;
if ((xmax != 0) && (xmin != 0))
  {
    if (ymax > ymin)       /* restore to different scales */
      xmin = xmin * pow(10., ymax - ymin);
    else if (ymax < ymin)
      xmax = xmax * pow(10., ymin - ymax);
  }
if (graphinfo->xaxis_layout.nsig_fig == Xg3dSig2)  /* undo sig. digit shift */
  {
    xmin = xmin / 10;
    xmax = xmax / 10;
  }
if (graphinfo->xaxis_layout.nsig_fig == Xg3dSig3)
  {
    xmin = xmin / 100;
    xmax = xmax / 100;
  }
graphinfo->xaxis_layout.min = xmin * pow(10., ymin); /* restore power of 10 */
graphinfo->xaxis_layout.max = xmax * pow(10., ymax);

inc = graphinfo->yaxis_layout.increment - Xg3dLast0;
sprintf(buf, "%le", graphinfo->yaxis_layout.min);  /* get sci notation form */
sscanf(buf, "%[^e]", cnum);
sscanf(cnum, "%le", &ymin);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%le", &xmin);
sprintf(buf, "%le", graphinfo->yaxis_layout.max);
sscanf(buf, "%[^e]", cnum);
sscanf(cnum, "%le", &ymax);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%le", &xmax);
if (graphinfo->yaxis_layout.nsig_fig == Xg3dSig2) /* scale to proper sig. digit*/
  {
    ymin = ymin * 10;
    ymax = ymax * 10;
  }
if (graphinfo->yaxis_layout.nsig_fig == Xg3dSig3)
  {
    ymin = ymin * 100;
    ymax = ymax * 100;
  }
if ((ymax != 0) && (ymin != 0))
  {
    if (xmax > xmin)          /* adjust both to same scale */
      ymin = ymin / pow(10., xmax - xmin);
    else if (xmax < xmin)
      ymax = ymax / pow(10., xmin - xmax);
  }
ymin = floor(ymin * 1.00001 / inc) * inc;  /* move up or down to next level */
ymax = ceil(ymax / inc) * inc;
if ((ymax != 0) && (ymin != 0))
  {
    if (xmax > xmin)       /* restore to different scales */
      ymin = ymin * pow(10., xmax - xmin);
    else if (xmax < xmin)
      ymax = ymax * pow(10., xmin - xmax);
  }
if (graphinfo->yaxis_layout.nsig_fig == Xg3dSig2)  /* undo sig. digit shift */
  {
    ymin = ymin / 10;
    ymax = ymax / 10;
  }
if (graphinfo->yaxis_layout.nsig_fig == Xg3dSig3)
  {
    ymin = ymin / 100;
    ymax = ymax / 100;
  }
graphinfo->yaxis_layout.min = ymin * pow(10., xmin); /* restore power of 10 */
graphinfo->yaxis_layout.max = ymax * pow(10., xmax);

/************** include zero if nearby *******************/
xmax = graphinfo->xaxis_layout.max - graphinfo->xaxis_layout.min;
if (!(graphinfo->xaxis_layout.user_set_flag & Xg3dUmin))
  {
    if ((graphinfo->xaxis_layout.min > 0) &&
	(graphinfo->xaxis_layout.min < xmax/2.))
      graphinfo->xaxis_layout.min = 0.;
  }
if (!(graphinfo->xaxis_layout.user_set_flag & Xg3dUmax))
  {
    if ((graphinfo->xaxis_layout.max < 0) &&
	(-graphinfo->xaxis_layout.max < xmax/2.))
      graphinfo->xaxis_layout.max = 0.;
  }

ymax = graphinfo->yaxis_layout.max - graphinfo->yaxis_layout.min;
if (!(graphinfo->yaxis_layout.user_set_flag & Xg3dUmin))
  {
    if ((graphinfo->yaxis_layout.min > 0) &&
	(graphinfo->yaxis_layout.min < ymax/2.))
      graphinfo->yaxis_layout.min = 0.;
  }
if (!(graphinfo->yaxis_layout.user_set_flag & Xg3dUmax))
  {
    if ((graphinfo->yaxis_layout.max < 0) &&
	(-graphinfo->yaxis_layout.max < ymax/2.))
      graphinfo->yaxis_layout.max = 0.;
  }
#endif

Xg3d_drawGraph(graphinfo);
}
/****************************************/
Xg3d_drawGraph(graph)
     Xg3dGraphInfo
       *graph;
{
  Arg
    args[15];
  long int
    value;
  DEV_COORD
    nx,
    ny,
    xd,
    yd;
  int
    i,
    j,
    numx_points,
    numy_points,
    xmin,
    xmax,
    ymin,
    ymax,
    nxn,
    nyn,
    width,
    height,
    exp,
    nargs;
  XWindowAttributes
    attrib;
  Pixmap
    thePixmap;
  XImage
    *theImage;
  XPoint
    *point_list;
  XGCValues
    gc_val;
  double
    x,
    y,
    z,
    dx,
    hoff,
    scale_fac,
    min,
    max,
    inc;
  char
    *chrptr,
    buf[80],
    buf2[20],
    format[10];
/**********************/
if (graph->cursor_status == 1)  /* cursor is on */
  {
    Xg3d_delcurs(graph);
    graph->cursor_status = 1;
  }
if (graph->draw_area != NULL)   /* only NULL first time */
  {
    XtDestroyWidget(graph->draw_area);
  }
nargs = 0;
XtSetArg(args[nargs], XmNborderWidth, 0); ++nargs;
XtSetArg(args[nargs], XmNbackground, graph->background_color); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, graph->menu_bar); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
graph->draw_area = XmCreateDrawingArea(graph->form, "draw area", args, nargs);
XtManageChild(graph->draw_area);

XtRealizeWidget(graph->app_shell);
/*XtAddCallback(graph->draw_area,XmNresizeCallback, Xg3d_redrawExpose, NULL);*/
XtAddCallback(graph->draw_area, XmNexposeCallback, Xg3d_redrawExpose, graph);
x = (double) (graph->xaxis_layout.win_max - graph->xaxis_layout.win_min) * .2;
y = (double) (graph->yaxis_layout.win_max - graph->yaxis_layout.win_min) * .2;
Xg3dSetworld(graph->xaxis_layout.win_min - x, graph->yaxis_layout.win_min - y,
	 graph->xaxis_layout.win_max+x, graph->yaxis_layout.win_max+y, graph);
graph->theGC = Xg3dCreateGC(graph);
dx = graph->hoff * tan(graph->theta * 3.14159 / 180.);
hoff = graph->hoff;
scale_fac = graph->scale_fac;
numx_points = graph->xaxis_layout.max - graph->xaxis_layout.min;
numy_points = graph->yaxis_layout.max - graph->yaxis_layout.min;
if (graph->dataset[0].line_status == Xg3dYLine)
  point_list = (XPoint *) calloc(numy_points*2, sizeof(XPoint));
else
  point_list = (XPoint *) calloc(numx_points*2, sizeof(XPoint));

/*************** get current sizes ******************/
nargs = 0;
XtSetArg(args[nargs], XmNwidth, &nx); ++nargs;
XtSetArg(args[nargs], XmNheight, &ny); ++nargs;
XtGetValues(graph->draw_area, args, nargs);
graph->nxdraw = nx;
graph->nydraw = ny;

/***************** draw graph box ****************/
XSetForeground(XtDisplay(graph->draw_area), graph->theGC, graph->axis_color);
Xg3dDrawLine(graph->xaxis_layout.win_min, graph->yaxis_layout.win_min,
	   graph->xaxis_layout.win_min, graph->yaxis_layout.win_max, graph);
Xg3dDrawLine(graph->xaxis_layout.win_min, graph->yaxis_layout.win_max,
	   graph->xaxis_layout.win_max, graph->yaxis_layout.win_max, graph);
Xg3dDrawLine(graph->xaxis_layout.win_max, graph->yaxis_layout.win_max,
	   graph->xaxis_layout.win_max, graph->yaxis_layout.win_min, graph);
Xg3dDrawLine(graph->xaxis_layout.win_max, graph->yaxis_layout.win_min,
	   graph->xaxis_layout.win_min, graph->yaxis_layout.win_min, graph);

/******** labels **********/
XSetForeground(XtDisplay(graph->draw_area), graph->theGC, graph->text_color);
nx = graph->x_axis_coord.x * graph->nxdraw / 100.;
ny = (100. - graph->x_axis_coord.y) * graph->nydraw / 100.;
nx -= XTextWidth(graph->x_axis_font.font, graph->x_axis_text,
       strlen(graph->x_axis_text)) / 2; /* x coord is center of string */
/*ny -= graph->x_axis_font.font->ascent + graph->x_axis_font.font->descent;*/
XSetFont(XtDisplay(graph->draw_area), graph->theGC,
	 graph->x_axis_font.font->fid);
XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	    graph->theGC, nx, ny, graph->x_axis_text,
	    strlen(graph->x_axis_text));

if (strlen(graph->y_axis_text) > 0)
  {
    width = XTextWidth(graph->y_axis_font.font, graph->y_axis_text,
		       strlen(graph->y_axis_text));
height = graph->y_axis_font.font->ascent + graph->y_axis_font.font->descent;
XGetWindowAttributes(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		     &attrib);
thePixmap = XCreatePixmap(XtDisplay(graph->draw_area),
			  XtWindow(graph->draw_area), width, height,
			  attrib.depth);
XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
	       graph->background_color);
XFillRectangle(XtDisplay(graph->draw_area), thePixmap, graph->theGC, 0, 0,
	       width, height);
XSetForeground(XtDisplay(graph->draw_area), graph->theGC, graph->text_color);
XSetFont(XtDisplay(graph->draw_area), graph->theGC,
	 graph->y_axis_font.font->fid);
XDrawString(XtDisplay(graph->draw_area), thePixmap, graph->theGC, 0,
	    height - graph->y_axis_font.font->descent, graph->y_axis_text,
	    strlen(graph->y_axis_text));
theImage = XGetImage(XtDisplay(graph->draw_area), thePixmap, 0, 0,
		     width, height, 15, XYPixmap);
nx = graph->y_axis_coord.x * graph->nxdraw / 100.;
ny = graph->y_axis_coord.y * graph->nydraw / 100.;
ny += XTextWidth(graph->y_axis_font.font, graph->y_axis_text,
       strlen(graph->y_axis_text)) / 2; /* y coord is center of string */
nx += graph->y_axis_font.font->ascent + graph->y_axis_font.font->descent;
nyn = ny;
for (i=0; i<width; ++i)
  {
    nxn = nx;
    for (j=height-1; j>=0; --j)
      {
	value = XGetPixel(theImage, i, j);
	XSetForeground(XtDisplay(graph->draw_area), graph->theGC, value);
	XDrawPoint(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		   graph->theGC, nxn, nyn);
	--nxn;
      }
    --nyn;
  }
    XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
		   graph->text_color);
    XFreePixmap(XtDisplay(graph->draw_area), thePixmap);
  }

nx = graph->title_coord.x * graph->nxdraw / 100.;
ny = graph->title_coord.y * graph->nydraw / 100.;
nx -= XTextWidth(graph->title_font.font, graph->title_text,
       strlen(graph->title_text)) / 2; /* x coord is center of string */
ny += graph->title_font.font->ascent + graph->title_font.font->descent;
XSetFont(XtDisplay(graph->draw_area), graph->theGC,
	 graph->title_font.font->fid);
XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	    graph->theGC, nx, ny, graph->title_text,
	    strlen(graph->title_text));

nx = graph->subtitle_coord.x * graph->nxdraw / 100.;
ny = graph->subtitle_coord.y * graph->nydraw / 100.;
nx -= XTextWidth(graph->subtitle_font.font, graph->subtitle_text,
       strlen(graph->subtitle_text)) / 2; /* x coord is center of string */
ny += graph->subtitle_font.font->ascent
  + graph->subtitle_font.font->descent;
XSetFont(XtDisplay(graph->draw_area), graph->theGC,
	 graph->subtitle_font.font->fid);
XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	    graph->theGC, nx, ny, graph->subtitle_text,
	    strlen(graph->subtitle_text));

#ifdef JUNK
XSetFont(XtDisplay(graph->draw_area),graph->theGC,graph->scale_font.font->fid);
XSetForeground(XtDisplay(graph->draw_area), graph->theGC, graph->axis_color);

max = fabs(graph->xaxis_layout.max);
min = fabs(graph->xaxis_layout.min);
if (max > min)
  sprintf(buf, "%le", max);
else
  sprintf(buf, "%le", min);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%d", &exp);
sprintf(buf, "x data = graph * 10^%d", exp);
if (graph->label_type == Xg3dSci_label)
  {
    nx = graph->xaxis_layout.scale_loc.x * graph->nxdraw / 100.;
    ny = graph->xaxis_layout.scale_loc.y * graph->nydraw / 100.;
    if (nx+XTextWidth(graph->scale_font.font,buf,strlen(buf)) > graph->nxdraw)
      nx = graph->nxdraw
	- XTextWidth(graph->scale_font.font, buf,strlen(buf)) -5;
    ny += graph->scale_font.font->ascent + graph->scale_font.font->descent;
    XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		graph->theGC, nx, ny, buf, strlen(buf));
  }

/***** x-axis numbering *******/
sprintf(format, "%%%d.%dlf", graph->xaxis_layout.nsig_fig - Xg3dSig0 + 1,
	graph->xaxis_layout.nsig_fig - Xg3dSig0 - 1);
if (graph->label_type == Xg3dNo_label)
    sprintf(format, "%%lg");
y = graph->yaxis_layout.min;
min = graph->xaxis_layout.min;
max = graph->xaxis_layout.max;
inc = pow(10., (double) (exp - (graph->xaxis_layout.nsig_fig - Xg3dSig0 - 1)));
inc = (double) (graph->xaxis_layout.increment - Xg3dLast0) * inc;
for (x=min; x<=max; x+=inc)
    {
      Xg3dMapwtod(x, y, &nx, &ny, graph);
      XDrawLine(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		graph->theGC, nx, ny, nx, ny + 5);
      z = x;
      if (graph->label_type == Xg3dSci_label)
	  z = z / pow(10., (double) exp);
      else
	if (fabs(z) < inc / 10.)
	  z = 0.;
      sprintf(buf, format, z);
      sscanf(buf, "%s", buf2);
      ny += 5 + graph->scale_font.font->ascent
	+ graph->scale_font.font->descent;
      nx -= XTextWidth(graph->scale_font.font, buf2,
		       strlen(buf2)) / 2; /* x coord is center of string */
      XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		  graph->theGC, nx, ny,	buf2, strlen(buf2));
    }

max = fabs(graph->yaxis_layout.max);
min = fabs(graph->yaxis_layout.min);
if (max > min)
  sprintf(buf, "%le", max);
else
  sprintf(buf, "%le", min);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%d", &exp);
sprintf(buf, "y data = graph * 10^%d", exp);
if (graph->label_type == Xg3dSci_label)
  {
    nx = graph->yaxis_layout.scale_loc.x * graph->nxdraw / 100.;
    ny = graph->yaxis_layout.scale_loc.y * graph->nydraw / 100.;
    if (nx+XTextWidth(graph->scale_font.font,buf,strlen(buf)) > graph->nxdraw)
      nx = graph->nxdraw
	- XTextWidth(graph->scale_font.font, buf,strlen(buf)) -5;
    ny += graph->scale_font.font->ascent + graph->scale_font.font->descent;
    XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		graph->theGC, nx, ny, buf, strlen(buf));
  }

/***** y-axis numbering *******/
sprintf(format, "%%%d.%dlf", graph->yaxis_layout.nsig_fig - Xg3dSig0 + 1,
	graph->yaxis_layout.nsig_fig - Xg3dSig0 - 1);
if (graph->label_type == Xg3dNo_label)
    sprintf(format, "%%lg");
x = graph->xaxis_layout.min;
min = graph->yaxis_layout.min;
max = graph->yaxis_layout.max;
inc = pow(10., (double) (exp - (graph->yaxis_layout.nsig_fig - Xg3dSig0 - 1)));
inc = (double) (graph->yaxis_layout.increment - Xg3dLast0) * inc;
for (y=min; y<=max; y+=inc)
    {
      Xg3dMapwtod(x, y, &nx, &ny, graph);
      XDrawLine(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		graph->theGC, nx-1, ny,	nx - 5, ny);
      z = y;
      if (graph->label_type == Xg3dSci_label)
	z = z / pow(10., (double) exp);
      else
	if (fabs(z) < inc / 10.)
	  z = 0.;
      sprintf(buf, format, z);
      sscanf(buf, "%s", buf2);
      ny += (graph->scale_font.font->ascent) / 2;
/*	+ graph->scale_font.font->descent) / 2;*/
      nx -= (5 + XTextWidth(graph->scale_font.font, buf2, strlen(buf2)));
      XDrawString(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		  graph->theGC, nx, ny,	buf2, strlen(buf2));
    }
#endif

/*********** draw data ***************/
nargs = graph->dataset[0].point_size - Xg3dSize0;
XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
	       graph->dataset[0].datacolor);
xmin = (int) graph->xaxis_layout.min;
xmax = (int) graph->xaxis_layout.max;
ymin = (int) graph->yaxis_layout.min;
ymax = (int) graph->yaxis_layout.max;
for (j=ymax-1; j>=ymin; --j)
  {
    switch (graph->dataset[0].point_type)
      {
      case Xg3dPoint:
	for (i=xmin; i<xmax; ++i)
	    Xg3dDrawPoint(i + dx*(j - ymin),
			(j-ymin)*hoff + Xg3dScale_Func(i, j, graph)
		        * scale_fac, graph);
	break;
      case Xg3dCircle:
	for (i=xmin; i<xmax; ++i)
	    Xg3dDrawCircle(i + dx*(j - ymin),
			(j-ymin)*hoff + Xg3dScale_Func(i, j, graph)
		        * scale_fac, nargs, graph);
	break;
      case Xg3dSquare:
	for (i=xmin; i<xmax; ++i)
	    Xg3dDrawSquare(i + dx*(j - ymin),
			(j-ymin)*hoff + Xg3dScale_Func(i, j, graph)
		        * scale_fac, nargs, graph);
	break;
      case Xg3dBar: /** nothing, blank **/
	break;
      case Xg3dTriangle:
	for (i=xmin; i<xmax; ++i)
	    Xg3dDrawTriangle(i + dx*(j - ymin),
			(j-ymin)*hoff + Xg3dScale_Func(i, j, graph)
		        * scale_fac, nargs, graph);
	break;
      default:
	break;
      }

    if (graph->dataset[0].line_status == Xg3dLine)
      {
	if (j > ymin)
	  {
	    for (i=xmin; i<xmax; ++i)
	      {
		Xg3dMapwtod(i + dx * (j - ymin), (j-ymin)*hoff
			    + Xg3dScale_Func(i, j, graph) * scale_fac,
			    &xd, &yd, graph);
		(point_list + i-xmin)->x = xd;
		(point_list + i-xmin)->y = yd;
		Xg3dMapwtod(i + dx * (j - 1 - ymin), (j-1-ymin)*hoff
			    + Xg3dScale_Func(i, j-1, graph) * scale_fac,
			    &xd, &yd, graph);
		(point_list + 2*numx_points - 1 - (i-xmin))->x = xd;
		(point_list + 2*numx_points - 1 - (i-xmin))->y = yd;
	      }
	    XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
			   graph->background_color);
	    XFillPolygon(XtDisplay(graph->draw_area),
			 XtWindow(graph->draw_area), graph->theGC,
			 point_list, 2*numx_points, Nonconvex,CoordModeOrigin);
	    XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
			   graph->dataset[0].datacolor);
/*	    gc_val.function = GXand;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	    XDrawLines(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		       graph->theGC, point_list, numx_points, CoordModeOrigin);
/*	    gc_val.function = GXcopy;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	  }
	else
	  {
/*	    gc_val.function = GXand;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	    XDrawLines(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		       graph->theGC, point_list + numx_points, numx_points,
		       CoordModeOrigin);
/*	    gc_val.function = GXcopy;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	  }
      }

    if (graph->dataset[0].line_status == Xg3dMesh)
      {
	if (j > ymin)
	  {
	    for (i=xmin; i<xmax; ++i)
	      {
		Xg3dMapwtod(i + dx * (j - ymin), (j-ymin)*hoff
			    + Xg3dScale_Func(i, j, graph) * scale_fac,
			    &xd, &yd, graph);
		(point_list + i-xmin)->x = xd;
		(point_list + i-xmin)->y = yd;
		Xg3dMapwtod(i + dx * (j - 1 - ymin), (j-1-ymin)*hoff
			    + Xg3dScale_Func(i, j-1, graph) * scale_fac,
			    &xd, &yd, graph);
		(point_list + 2*numx_points - 1 - (i-xmin))->x = xd;
		(point_list + 2*numx_points - 1 - (i-xmin))->y = yd;
	      }
	    XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
			   graph->background_color);
	    XFillPolygon(XtDisplay(graph->draw_area),
			 XtWindow(graph->draw_area), graph->theGC,
			 point_list, 2*numx_points, Nonconvex,CoordModeOrigin);
	    XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
			   graph->dataset[0].datacolor);
/*	    gc_val.function = GXand;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	    XDrawLines(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		       graph->theGC, point_list, numx_points, CoordModeOrigin);
	    for (i=xmin; i<xmax; ++i)
	      {
		XDrawLine(XtDisplay(graph->draw_area),
			  XtWindow(graph->draw_area), graph->theGC,
			  (point_list + i-xmin)->x, (point_list + i-xmin)->y,
			  (point_list + 2*numx_points - 1 - (i-xmin))->x,
			  (point_list + 2*numx_points - 1 - (i-xmin))->y);
	      }
/*	    gc_val.function = GXcopy;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	  }
	else
	  {
/*	    gc_val.function = GXand;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	    XDrawLines(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		       graph->theGC, point_list + numx_points, numx_points,
		       CoordModeOrigin);
/*	    gc_val.function = GXcopy;
	    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		      &gc_val);*/
	  }
      }
  }

if (graph->dataset[0].line_status == Xg3dYLine)
  {
    for (i=xmin; i<xmax-1; ++i)
      {
	for (j=ymin; j<ymax; ++j)
	  {
	    Xg3dMapwtod(i + dx * (j - ymin), (j-ymin)*hoff
			+ Xg3dScale_Func(i, j, graph) * scale_fac,
			&xd, &yd, graph);
	    (point_list + j-ymin)->x = xd;
	    (point_list + j-ymin)->y = yd;
	    Xg3dMapwtod(i + 1 + dx * (j - ymin), (j-ymin)*hoff
			+ Xg3dScale_Func(i+1, j, graph) * scale_fac,
			&xd, &yd, graph);
	    (point_list + 2*numy_points - 1 - (j-ymin))->x = xd;
	    (point_list + 2*numy_points - 1 - (j-ymin))->y = yd;
	  }
	XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
		       graph->background_color);
	XFillPolygon(XtDisplay(graph->draw_area),
		     XtWindow(graph->draw_area), graph->theGC,
		     point_list, 2*numy_points, Nonconvex, CoordModeOrigin);
	XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
		       graph->dataset[0].datacolor);
/*	gc_val.function = GXand;
	XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		  &gc_val);*/
	XDrawLines(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		   graph->theGC, point_list, numy_points, CoordModeOrigin);
/*	gc_val.function = GXcopy;
	XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
		  &gc_val);*/
      }
/*    gc_val.function = GXand;
    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
	      &gc_val);*/
    XDrawLines(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	       graph->theGC, point_list + numy_points, numy_points,
	       CoordModeOrigin);
/*    gc_val.function = GXcopy;
    XChangeGC(XtDisplay(graph->draw_area), graph->theGC, GCFunction,
	      &gc_val);*/
  }

free(point_list);

if (graph->cursor_status == 1)  /* if cursor is on, redraw it */
  {
    Xg3d_initcurs(graph);
  }
XFlush(XtDisplay(graph->draw_area));
}
/********************************************/
void Xg3dDrawPoint(xw, yw, graph)
     double
       xw,
       yw;
     Xg3dGraphInfo
       *graph;
{
  double
    min,
    max;
  DEV_COORD
    xd,
    yd;
min = graph->xaxis_layout.win_min;
max = graph->xaxis_layout.win_max;
if ((xw < min) || (xw > max))
  return;
min = graph->yaxis_layout.win_min;
max = graph->yaxis_layout.win_max;
if ((yw < min) || (yw > max))
  return;
Xg3dMapwtod(xw, yw, &xd, &yd, graph);
XDrawPoint(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	   graph->theGC, xd, yd);
}
/*************************/
void Xg3dDrawCircle(xw, yw, size, graph)
     double
       xw,
       yw;
     int
       size;
     Xg3dGraphInfo
       *graph;
{
  double
    min,
    max;
  DEV_COORD
    xd,
    yd;
min = graph->xaxis_layout.win_min;
max = graph->xaxis_layout.win_max;
if ((xw < min) || (xw > max))
  return;
min = graph->yaxis_layout.win_min;
max = graph->yaxis_layout.win_max;
if ((yw < min) || (yw > max))
  return;
Xg3dMapwtod(xw, yw, &xd, &yd, graph);
XDrawArc(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	 graph->theGC, xd-size/2, yd-size/2, size, size, 0, 23040);
}
/*************************/
void Xg3dDrawSquare(xw, yw, size, graph)
     double
       xw,
       yw;
     int
       size;
     Xg3dGraphInfo
       *graph;
{
  double
    min,
    max;
  DEV_COORD
    xd,
    yd;
min = graph->xaxis_layout.win_min;
max = graph->xaxis_layout.win_max;
if ((xw < min) || (xw > max))
  return;
min = graph->yaxis_layout.win_min;
max = graph->yaxis_layout.win_max;
if ((yw < min) || (yw > max))
  return;
Xg3dMapwtod(xw, yw, &xd, &yd, graph);
XDrawRectangle(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	       graph->theGC, xd-size/2, yd-size/2, size, size);
}
/*************************/
void Xg3dDrawBar(xw, yw, graph)
     double
       xw,
       yw;
     Xg3dGraphInfo
       *graph;
{
  double
    min,
    max;
  DEV_COORD
    xd,
    yd,
    dy;
min = graph->yaxis_layout.min;
Xg3dMapwtod(xw, min, &xd, &dy, graph);
min = graph->xaxis_layout.min;
max = graph->xaxis_layout.max;
if ((xw < min) || (xw > max))
  return;
min = graph->yaxis_layout.min;
max = graph->yaxis_layout.max;
if ((yw < min) || (yw > max))
  return;
Xg3dMapwtod(xw, yw, &xd, &yd, graph);
dy -= yd;
XFillRectangle(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	       graph->theGC, xd-2, yd, 4, dy); /* x, y, width, height, where
					    x, y are the upper left corner
					    and upper left is 0, 0 for wind */
}
/*************************/
void Xg3dDrawTriangle(xw, yw, size, graph)
     double
       xw,
       yw;
     int
       size;
     Xg3dGraphInfo
       *graph;
{
  double
    min,
    max;
  DEV_COORD
    xd,
    yd;
min = graph->xaxis_layout.win_min;
max = graph->xaxis_layout.win_max;
if ((xw < min) || (xw > max))
  return;
min = graph->yaxis_layout.win_min;
max = graph->yaxis_layout.win_max;
if ((yw < min) || (yw > max))
  return;
Xg3dMapwtod(xw, yw, &xd, &yd, graph);
XDrawLine(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	  graph->theGC, xd-size/2, yd+size/2, xd+size/2, yd+size/2);
XDrawLine(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	  graph->theGC, xd+size/2, yd+size/2, xd, yd-size/2);
XDrawLine(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	  graph->theGC, xd, yd-size/2, xd-size/2, yd+size/2);
}
/*********************/
void Xg3dDrawLine(xw1, yw1, xw2, yw2, graph)
     double
       xw1,
       yw1,
       xw2,
       yw2;
     Xg3dGraphInfo
       *graph;
{
  double
    min,
    max;
  DEV_COORD
    xd1,
    yd1,
    xd2,
    yd2;
min = graph->xaxis_layout.win_min;
max = graph->xaxis_layout.win_max;
if ((xw1 < min) || (xw1 > max) || (xw2 < min) || (xw2 > max))
  return;
min = graph->yaxis_layout.win_min;
max = graph->yaxis_layout.win_max;
if ((yw1 < min) || (yw1 > max) || (yw2 < min) || (yw2 > max))
  return;
Xg3dMapwtod(xw1, yw1, &xd1, &yd1, graph);
Xg3dMapwtod(xw2, yw2, &xd2, &yd2, graph);
XDrawLine(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	  graph->theGC, xd1, yd1, xd2, yd2);
}
/************************************/
GC Xg3dCreateGC(graph)
     Xg3dGraphInfo
       *graph;
{
  XGCValues
    theGCValues;
  GC
    theGC;
theGC = (GC) XCreateGC(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		  (unsigned long) NULL, &theGCValues);
if (theGC == NULL)
  {
    fprintf(stderr, "Error creating GC: Aborted\n");
    exit(1);
  }
else
    {
      XSetForeground(XtDisplay(graph->draw_area), theGC, graph->text_color);
      XSetBackground(XtDisplay(graph->draw_area), theGC,
		     graph->background_color);
      return(theGC);
    }
}
/*************************************/
void Xg3dSetworld(xll, yll, xur, yur, graph)
     double
       xll,
       yll,
       xur,
       yur;
     Xg3dGraphInfo
       *graph;
{
graph->sw_xll = xll;
graph->sw_yll = yll;
graph->sw_xur = xur;
graph->sw_yur = yur;
}
/***************************/
void Xg3dMapwtod(xw, yw, xd, yd, graph)
     double
       xw,
       yw;
     DEV_COORD
       *xd,
       *yd;
     Xg3dGraphInfo
       *graph;
{
*xd = (int) floor((xw - graph->sw_xll) / (graph->sw_xur - graph->sw_xll)
		  * graph->nxdraw + 0.5);
*yd = graph->nydraw - (int) floor((yw - graph->sw_yll)
	  / (graph->sw_yur - graph->sw_yll) * graph->nydraw + 0.5);
}
/**********************/
void Xg3dMapdtow(xd, yd, xw, yw, graph)
     int
       xd,
       yd;
     double
       *xw,
       *yw;
     Xg3dGraphInfo
       *graph;
{
*xw = graph->sw_xll + (graph->sw_xur - graph->sw_xll) *
  (double) (xd) / (double) (graph->nxdraw);
*yw = (double) (graph->nydraw - yd) / (double) (graph->nydraw)
  * (graph->sw_yur - graph->sw_yll) + graph->sw_yll;
}
/**********************/
void Xg3d_redrawExpose(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
if (graph->redraw == 0)  /** if 1st time through, set flag & redraw graph **/
  {
    graph->redraw = 1;
    Xg3d_createGraph(graph);
  }
else            /** if 2nd time through, reset flag & return **/
  {
    graph->redraw = 0;
    return;
  }
}
/*************** Postscript Output File *******************/
void Xg3dPrint(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  FILE
    *fp;
  Xg3dGraphInfo
    *graph;
  long int
    time_sec;
  DEV_COORD
    x1d,
    y1d,
    x2d,
    y2d;
  int
    i,
    j,
    k,
    nx,
    ny,
    exp,
    num_points,
    xmin,
    ymin,
    xmax,
    ymax,
    x_font_size,
    y_font_size,
    title_font_size,
    subtitle_font_size,
    scale_font_size;
  double
    min,
    max,
    x,
    y,
    z,
    dx,
    hoff,
    scale_fac,
    inc;
  float
    size,
    scalex,
    scaley,
    page_long_dim,
    page_short_dim;
  char
    *chrptr,
    format[10],
    buf2[20],
    buf[80],
    x_font_name[80],
    y_font_name[80],
    title_font_name[80],
    subtitle_font_name[80],
    scale_font_name[80];
graph = (Xg3dGraphInfo *) client_data;
time(&time_sec);
x_font_size = 16;
y_font_size = 16;
title_font_size = 20;
subtitle_font_size = 18;
scale_font_size = 14;
page_long_dim = 11.;
page_short_dim = 8.5;
strcpy(x_font_name, "Helvetica");
strcpy(y_font_name, "Helvetica");
strcpy(title_font_name, "Helvetica-Bold");
strcpy(subtitle_font_name, "Helvetica-Bold");
strcpy(scale_font_name, "Helvetica");

fp = NULL;
fp = fopen("Xg3d.psc", "w");
if (fp == NULL)
  {
    fprintf(stderr, "Failed to open Xg3d.psc output file\n");
    return;
  }
/******* header ********/
fprintf(fp, "%%!PS-Adobe-1.0\n");
fprintf(fp, "%%%%Creator: Xg3dPlot program\n");
fprintf(fp, "%%%%Title: Graph\n");
fprintf(fp, "%%%%Creation Date: %s", ctime(&time_sec));
fprintf(fp, "%%%%BoundingBox: 0 0 611 791\n");
fprintf(fp, "%%%%End Comments\n");
fprintf(fp, "%%%%Pages: 1\n");
fprintf(fp, "%%%%End Prolog\n");
fprintf(fp, "%%%%Page: 1 1\n\n");
fprintf(fp, "%% For EPS usage:\n");
fprintf(fp, "%%   1) Add EPSF-1.2 at end of first line\n");
fprintf(fp, "%%   2) Switch BoundingBox coordinates to 0 0 791 611\n");
fprintf(fp, "%%   3) Comment out the two lines indicated just below\n");
fprintf(fp, "%%   4) remove showpage at end of this file\n");

/********* coordinate system **********/
fprintf(fp, "/inch {72 mul} def\n");
fprintf(fp, "%% comment out the next two lines for EPS usage\n");
fprintf(fp, "%f inch 0 inch translate\n", page_short_dim);
fprintf(fp, "90 rotate\n");

/************** labels **************/
fprintf(fp, "/%s findfont %d scalefont setfont\n", x_font_name, x_font_size);
fprintf(fp, "%f inch %f inch moveto\n",     /** x-axis **/
	(graph->x_axis_coord.x / 100. * page_long_dim),
	(graph->x_axis_coord.y / 100. * page_short_dim));
fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
	graph->x_axis_text, x_font_size);
fprintf(fp, "/%s findfont %d scalefont setfont\n", y_font_name, y_font_size);
fprintf(fp, "%f inch %f inch moveto\n",     /** y-axis **/
	(graph->y_axis_coord.x / 100. * page_long_dim),
	(graph->y_axis_coord.y / 100. * page_short_dim));
fprintf(fp, "gsave 90 rotate\n");
fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
	graph->y_axis_text, -y_font_size);
fprintf(fp, "grestore\n");
fprintf(fp, "/%s findfont %d scalefont setfont\n", title_font_name,
	title_font_size);
fprintf(fp, "%f inch %f inch moveto\n",     /** title **/
	(graph->title_coord.x / 100. * page_long_dim),
	(8.5 - graph->title_coord.y / 100. * page_short_dim));
fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
	graph->title_text, -title_font_size);
if (strlen(graph->subtitle_text) > 0)
  {
    fprintf(fp, "/%s findfont %d scalefont setfont\n", subtitle_font_name,
	    subtitle_font_size);
    fprintf(fp, "%f inch %f inch moveto\n",  /** sub-title **/
	    (graph->subtitle_coord.x / 100. * page_long_dim),
	    (8.5 - graph->subtitle_coord.y / 100. * page_short_dim));
    fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n\n",
	    graph->subtitle_text, -subtitle_font_size);
  }

/************* axes box **************/
nx = graph->nxdraw;
ny = graph->nydraw;
scalex = page_long_dim * 72. / (float) nx;
scaley = page_short_dim * 72. / (float) ny;
fprintf(fp, "%f %f scale\n", scalex, scaley);
Xg3dMapwtod((double) graph->xaxis_layout.win_min,
	    (double) graph->yaxis_layout.win_min, &x1d, &y1d, graph);
Xg3dMapwtod((double) graph->xaxis_layout.win_min,
	    (double) graph->yaxis_layout.win_max, &x2d, &y2d, graph);
y1d = ny - y1d;
y2d = ny - y2d;
fprintf(fp, "%d %d moveto %d %d lineto\n", x1d, y1d, x2d, y2d);
Xg3dMapwtod((double) graph->xaxis_layout.win_max,
	    (double) graph->yaxis_layout.win_max, &x1d, &y1d, graph);
y1d = ny - y1d;
fprintf(fp, "%d %d lineto\n", x1d, y1d);
Xg3dMapwtod((double) graph->xaxis_layout.win_max,
	    (double) graph->yaxis_layout.win_min, &x1d, &y1d, graph);
y1d = ny - y1d;
fprintf(fp, "%d %d lineto closepath stroke\n\n", x1d, y1d);

#ifdef JUNK
/************* scales ************/
fprintf(fp, "/%s findfont %d scalefont setfont\n", scale_font_name,
	(int) (scale_font_size / scalex));
max = fabs(graph->xaxis_layout.max);    /** x axis scale factor **/
min = fabs(graph->xaxis_layout.min);
if (max > min)
  sprintf(buf, "%le", max);
else
  sprintf(buf, "%le", min);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%d", &exp);
sprintf(buf, "x data = graph * 10^%d", exp);
if (graph->label_type == Xg3dSci_label)
  {
    x = graph->xaxis_layout.scale_loc.x * nx / 100.;
    y = graph->xaxis_layout.scale_loc.y * ny / 100.;
    fprintf(fp, "%lg (%s) stringwidth pop add %d gt\n", x, buf, nx);
    fprintf(fp, "  { %f %f moveto\n", x, ny - y);
    fprintf(fp, "    (%s) dup stringwidth\n", buf);
    fprintf(fp, "    pop neg %d rmoveto show }\n",
	      (int) (-2. * scale_font_size / scalex));
    fprintf(fp, "  { %f %f moveto\n", x, ny - y);
    fprintf(fp, "    0 %d rmoveto (%s) show } ifelse\n",
	      (int) (-2. * scale_font_size / scalex), buf);
  }

/***** x-axis numbering *******/
sprintf(format, "%%%d.%dlf", graph->xaxis_layout.nsig_fig - Xg3dSig0 + 1,
	graph->xaxis_layout.nsig_fig - Xg3dSig0 - 1);
if (graph->label_type == Xg3dNo_label)
    sprintf(format, "%%lg");
y = graph->yaxis_layout.min;
min = graph->xaxis_layout.min;
max = graph->xaxis_layout.max;
inc = pow(10., (double) (exp - (graph->xaxis_layout.nsig_fig - Xg3dSig0 - 1)));
inc = (double) (graph->xaxis_layout.increment - Xg3dLast0) * inc;
for (x=min; x<=max; x+=inc)
    {
      Xg3dMapwtod(x, y, &x1d, &y1d, graph);
      y1d = ny - y1d;
      fprintf(fp, "%d %d moveto 0 -5 rlineto stroke\n", x1d, y1d);
      z = x;
      if (graph->label_type == Xg3dSci_label)
	  z = z / pow(10., (double) exp);
      else
	if (fabs(z) < inc / 10.)
	  z = 0.;
      sprintf(buf, format, z);
      sscanf(buf, "%s", buf2);
      fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d, y1d-5, buf2);
      fprintf(fp, "pop neg 2 div %d rmoveto show\n", 
	      (int) (-scale_font_size / scalex));
    }

max = fabs(graph->yaxis_layout.max);   /** y axis scale factor **/
min = fabs(graph->yaxis_layout.min);
if (max > min)
  sprintf(buf, "%le", max);
else
  sprintf(buf, "%le", min);
chrptr = strchr(buf, 'e');
++chrptr;
sscanf(chrptr, "%d", &exp);
sprintf(buf, "y data = graph * 10^%d", exp);
if (graph->label_type == Xg3dSci_label)
  {
    x = graph->yaxis_layout.scale_loc.x * nx / 100.;
    y = graph->yaxis_layout.scale_loc.y * ny / 100.;
    fprintf(fp, "%lg (%s) stringwidth pop add %d gt\n", x, buf, nx);
    fprintf(fp, "  { %f %f moveto\n", x, ny - y);
    fprintf(fp, "    (%s) dup stringwidth\n", buf);
    fprintf(fp, "    pop neg %d rmoveto show }\n",
	      (int) (-2. * scale_font_size / scalex));
    fprintf(fp, "  { %f %f moveto\n", x, ny - y);
    fprintf(fp, "    0 %d rmoveto (%s) show } ifelse\n",
	      (int) (-2. * scale_font_size / scalex), buf);
  }

/***** y-axis numbering *******/
sprintf(format, "%%%d.%dlf", graph->yaxis_layout.nsig_fig - Xg3dSig0 + 1,
	graph->yaxis_layout.nsig_fig - Xg3dSig0 - 1);
if (graph->label_type == Xg3dNo_label)
    sprintf(format, "%%lg");
x = graph->xaxis_layout.min;
min = graph->yaxis_layout.min;
max = graph->yaxis_layout.max;
inc = pow(10., (double) (exp - (graph->yaxis_layout.nsig_fig - Xg3dSig0 - 1)));
inc = (double) (graph->yaxis_layout.increment - Xg3dLast0) * inc;
for (y=min; y<=max; y+=inc)
    {
      Xg3dMapwtod(x, y, &x1d, &y1d, graph);
      y1d = ny - y1d;
      fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n", x1d, y1d);
      z = y;
      if (graph->label_type == Xg3dSci_label)
	z = z / pow(10., (double) exp);
      else
	if (fabs(z) < inc / 10.)
	  z = 0.;
      sprintf(buf, format, z);
      sscanf(buf, "%s", buf2);
      fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d-6, y1d, buf2);
      fprintf(fp, "pop neg %d rmoveto show\n", 
	      (int) (-scale_font_size / scalex / 2.));
    }
#endif

/************* data ****************/
fprintf(fp, "\n.2 setlinewidth newpath\n");
dx = graph->hoff * tan(graph->theta * 3.14159 / 180.);
hoff = graph->hoff;
scale_fac = graph->scale_fac;
xmin = (int) graph->xaxis_layout.min;
xmax = (int) graph->xaxis_layout.max;
ymin = (int) graph->yaxis_layout.min;
ymax = (int) graph->yaxis_layout.max;
size = (graph->dataset[0].point_size - Xg3dSize0) / 2.;
for (j=ymax-1; j>=ymin; --j)
  {
    switch (graph->dataset[0].point_type)
      {
      case Xg3dPoint:
	for (i=xmin; i<xmax; ++i)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d 0 0 360 arc stroke\n", x1d, y1d);
	  }
	break;
      case Xg3dCircle:
	for (i=xmin; i<xmax; ++i)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d %f 0 360 arc stroke\n", x1d, y1d, size/2.);
	  }
	break;
      case Xg3dSquare:
	for (i=xmin; i<xmax; ++i)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%f %f moveto\n", x1d-size/2., y1d-size/2.);
	    fprintf(fp, "%f 0 rlineto 0 %f rlineto\n", size, size);
	    fprintf(fp, "%f 0 rlineto closepath stroke\n", -size);
	  }
	break;
      case Xg3dBar:
	break;
      case Xg3dTriangle:
	for (i=xmin; i<xmax; ++i)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%f %f moveto\n", x1d-size/2., y1d-size/2.);
	    fprintf(fp, "%f 0 rlineto %d %f lineto\n",
		    size, x1d, y1d+size/2.);
	    fprintf(fp, "closepath stroke\n");
	  }
	break;
      default:
	break;
      }

    if ((graph->dataset[0].line_status == Xg3dLine) ||
	(graph->dataset[0].line_status == Xg3dMesh))
      {
	if (j > ymin)
	  {
	    Xg3dMapwtod(xmin + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(xmin, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "newpath %d %d moveto\n", x1d, y1d);
	    for (i=xmin; i<xmax; ++i)
	      {
		Xg3dMapwtod(i + dx*(j - ymin),
		    (j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		    &x1d, &y1d, graph);
		y1d = ny - y1d;
		fprintf(fp, "%d %d lineto\n", x1d, y1d);
	      }
	    for (i=xmax-1; i>=xmin; --i)
	      {
		Xg3dMapwtod(i + dx*(j - 1 - ymin),
		   (j-1-ymin)*hoff + Xg3dScale_Func(i, j-1, graph) * scale_fac,
		   &x1d, &y1d, graph);
		y1d = ny - y1d;
		fprintf(fp, "%d %d lineto\n", x1d, y1d);
	      }
	    fprintf(fp, "closepath 1 setgray fill\n");

	    if (graph->dataset[0].line_status == Xg3dMesh) /* vertical lines */
	      {
		for (i=xmin; i<xmax; ++i)
		  {
		    Xg3dMapwtod(i + dx*(j - ymin),
		      (j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		      &x1d, &y1d, graph);
		    y1d = ny - y1d;
		    fprintf(fp, "newpath %d %d moveto\n", x1d, y1d);
		    Xg3dMapwtod(i + dx*(j - 1 - ymin),
		      (j-1-ymin)*hoff+Xg3dScale_Func(i, j-1,graph) *scale_fac,
		      &x1d, &y1d, graph);
		    y1d = ny - y1d;
		    fprintf(fp, "%d %d lineto 0 setgray stroke\n", x1d, y1d);
		  }
	      }
      }

	Xg3dMapwtod(xmin + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(xmin, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	y1d = ny - y1d;
	fprintf(fp, "newpath %d %d moveto\n", x1d, y1d);
	for (i=xmin+1; i<xmax; ++i)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d lineto\n", x1d, y1d);
	  }
	fprintf(fp, "0 setgray stroke\n\n");
      }
  }

if (graph->dataset[0].line_status == Xg3dYLine)
  {
    for (i=xmin; i<xmax-1; ++i)
      {
	Xg3dMapwtod(i, Xg3dScale_Func(i, ymin, graph) * scale_fac,
		    &x1d, &y1d, graph);
	y1d = ny - y1d;
	fprintf(fp, "newpath %d %d moveto\n", x1d, y1d);
	for (j=ymin+1; j<ymax-1; ++j)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d lineto\n", x1d, y1d);
	  }
	for (j=ymax-1; j>ymin; --j)
	  {
	    Xg3dMapwtod(i + 1 + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i+1, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d lineto\n", x1d, y1d);
	  }
	fprintf(fp, "closepath 1 setgray fill\n");

	Xg3dMapwtod(i, Xg3dScale_Func(i, ymin, graph) * scale_fac,
		    &x1d, &y1d, graph);
	y1d = ny - y1d;
	fprintf(fp, "newpath %d %d moveto\n", x1d, y1d);
	for (j=ymin+1; j<ymax-1; ++j)
	  {
	    Xg3dMapwtod(i + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(i, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d lineto\n", x1d, y1d);
	  }
	fprintf(fp, "0 setgray stroke\n\n");
      }

    Xg3dMapwtod(xmax-1, Xg3dScale_Func(xmax-1, ymin, graph) * scale_fac,
		&x1d, &y1d, graph);  /** last line **/
    y1d = ny - y1d;
    fprintf(fp, "newpath %d %d moveto\n", x1d, y1d);
    for (j=ymin+1; j<ymax-1; ++j)
      {
	Xg3dMapwtod(xmax-1 + dx*(j - ymin),
		(j-ymin)*hoff + Xg3dScale_Func(xmax-1, j, graph) * scale_fac,
		&x1d, &y1d, graph);
	y1d = ny - y1d;
	fprintf(fp, "%d %d lineto\n", x1d, y1d);
      }
    fprintf(fp, "0 setgray stroke\n\n");
  }

fprintf(fp, "showpage\n");
fclose(fp);
}
/****************************/
Xg3d_initcurs(graph)
     Xg3dGraphInfo
       *graph;
{
  int
    i;
  DEV_COORD
    x,
    y;
  XWindowAttributes
    attrib;
if (graph->cursor_status == 0) /* this is a turn on, not a redraw */
  {
    graph->cursPos = graph->xaxis_layout.min;
    graph->cursDset = graph->yaxis_layout.min;;
  }
graph->cursor_status = 1;  /* turn cursor on */
graph->cursWidth = 10;
graph->cursHeight = 10;
XGetWindowAttributes(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
		     &attrib);
graph->savePixmap = XCreatePixmap(XtDisplay(graph->draw_area),
				    XtWindow(graph->draw_area),
				    graph->cursWidth, graph->cursHeight,
				    attrib.depth);
graph->cursPixmap = XCreatePixmap(XtDisplay(graph->draw_area),
				    XtWindow(graph->draw_area),
				    graph->cursWidth, graph->cursHeight,
				    attrib.depth);
XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
	       graph->background_color);
XFillRectangle(XtDisplay(graph->draw_area), graph->cursPixmap, graph->theGC,
	       0, 0, graph->cursWidth, graph->cursHeight);
XSetForeground(XtDisplay(graph->draw_area), graph->theGC, graph->cursor_color);
for (i=0; i<graph->cursHeight; ++i)
  XDrawPoint(XtDisplay(graph->draw_area), graph->cursPixmap, graph->theGC,
	     (graph->cursWidth-1)/2, i);
for (i=0; i<graph->cursWidth; ++i)
  XDrawPoint(XtDisplay(graph->draw_area), graph->cursPixmap, graph->theGC, i,
	     (graph->cursHeight-1)/2);
Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
	    * (graph->cursDset - graph->yaxis_layout.min),
	    (graph->cursDset - graph->yaxis_layout.min) * graph->hoff
	    + Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
XCopyArea(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	  graph->savePixmap, graph->theGC,
	  x-(graph->cursWidth-1)/2, y-(graph->cursHeight-1)/2,
	  graph->cursWidth, graph->cursHeight, 0, 0);
/** create cursor position dialog **/
XtAddCallback(graph->draw_area, XmNinputCallback, Xg3d_processArrow, graph);
Xg3d_oncurs(graph);
}
/***************************/
Xg3d_delcurs(graph)
     Xg3dGraphInfo
       *graph;
{
  DEV_COORD
    x,
    y;
/** remove position pop-up **/
printf("\n");
graph->cursor_status = 0;  /* turn cursor off */
Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
	    * (graph->cursDset - graph->yaxis_layout.min),
	    (graph->cursDset - graph->yaxis_layout.min) * graph->hoff
	    + Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
XCopyArea(XtDisplay(graph->draw_area), graph->savePixmap,
	  XtWindow(graph->draw_area), graph->theGC, 0, 0, graph->cursWidth,
	  graph->cursHeight, x-(graph->cursWidth-1)/2,
	  y-(graph->cursHeight-1)/2);
XtRemoveAllCallbacks(graph->draw_area, XmNinputCallback);
XFreePixmap(XtDisplay(graph->draw_area), graph->savePixmap);
XFreePixmap(XtDisplay(graph->draw_area), graph->cursPixmap);
}
/***************************/
Xg3d_oncurs(graph)
     Xg3dGraphInfo
       *graph;
{
  DEV_COORD
    x,
    y;
/** update cursor readout **/
printf("row = %d; point = %d; z = %lg                             \r",
       graph->cursDset, graph->cursPos, Xg3dScale_Func(graph->cursPos,
       graph->cursDset, graph));
fflush(stdout);
Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
	    * (graph->cursDset - graph->yaxis_layout.min),
	    (graph->cursDset - graph->yaxis_layout.min) * graph->hoff
	    + Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
XCopyArea(XtDisplay(graph->draw_area), XtWindow(graph->draw_area),
	  graph->savePixmap, graph->theGC,
	  x-(graph->cursWidth-1)/2, y-(graph->cursHeight-1)/2,
	  graph->cursWidth, graph->cursHeight, 0, 0);
XSetFunction(XtDisplay(graph->draw_area), graph->theGC, GXand);
XCopyArea(XtDisplay(graph->draw_area), graph->cursPixmap,
	  XtWindow(graph->draw_area), graph->theGC, 0, 0, graph->cursWidth,
	  graph->cursHeight, x-(graph->cursWidth-1)/2,
	  y-(graph->cursHeight-1)/2);
XSetFunction(XtDisplay(graph->draw_area), graph->theGC, GXcopy);
XFlush(XtDisplay(graph->draw_area));
}
/***************************/
void Xg3d_processArrow(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  XmDrawingAreaCallbackStruct
    *theCallback;
  XEvent
    *theEvent;
  XKeyEvent
    *keyEvent;
  KeySym
    keysym;
  int
    i,
    n;
  DEV_COORD
    x,
    y;
  char
    KeyBuf[6];
#ifdef ENABLE_ROW_PLOT
  static Widget
    row_graph;
  static XgGraphInfo
    graphinfo;
  double
    *data;
  char
    title_buf[80];
#endif
graph = (Xg3dGraphInfo *) client_data;
if (graph->cursor_status == 0)  /* cursor is off, ignore & return */
  return;
theCallback = (XmDrawingAreaCallbackStruct *) call_data;
theEvent = theCallback->event;
if (theEvent->type != KeyPress) /* not interested, return */
  return;
keyEvent = (XKeyEvent *) theEvent;
XLookupString(keyEvent, KeyBuf, 4, &keysym, NULL);
switch (keysym)
  {
  case LEFTA:
    Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
		* (graph->cursDset - graph->yaxis_layout.min),
		(graph->cursDset - graph->yaxis_layout.min) * graph->hoff
		+ Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
    XCopyArea(XtDisplay(graph->draw_area), graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    --graph->cursPos;
    if (graph->cursPos < graph->xaxis_layout.min)
      graph->cursPos = graph->xaxis_layout.max - 1;
    Xg3d_oncurs(graph);
    break;
  case RIGHTA:
    Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
		* (graph->cursDset - graph->yaxis_layout.min),
		(graph->cursDset - graph->yaxis_layout.min) * graph->hoff
		+ Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
    XCopyArea(XtDisplay(graph->draw_area), graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    ++graph->cursPos;
    if (graph->cursPos > graph->xaxis_layout.max - 1)
      graph->cursPos = graph->xaxis_layout.min;
    Xg3d_oncurs(graph);
    break;
  case DOWNA:
    Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
		* (graph->cursDset - graph->yaxis_layout.min),
		(graph->cursDset - graph->yaxis_layout.min) * graph->hoff
		+ Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
    XCopyArea(XtDisplay(graph->draw_area), graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    --graph->cursDset;
    if (graph->cursDset < graph->yaxis_layout.min)
      graph->cursDset = graph->yaxis_layout.max - 1;
    Xg3d_oncurs(graph);
    break;
  case UPA:
    Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
		* (graph->cursDset - graph->yaxis_layout.min),
		(graph->cursDset - graph->yaxis_layout.min) * graph->hoff
		+ Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
    XCopyArea(XtDisplay(graph->draw_area), graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    ++graph->cursDset;
    if (graph->cursDset > graph->yaxis_layout.max - 1)
      graph->cursDset = graph->yaxis_layout.min;
    Xg3d_oncurs(graph);
    break;
  case HOME:
    Xg3dMapwtod(graph->cursPos + graph->hoff*tan(graph->theta * 3.14159 / 180.)
		* (graph->cursDset - graph->yaxis_layout.min),
		(graph->cursDset - graph->yaxis_layout.min) * graph->hoff
		+ Xg3dScale_Func(graph->cursPos, graph->cursDset, graph)
		* graph->scale_fac, &x, &y, graph);
    XCopyArea(XtDisplay(graph->draw_area), graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    graph->cursDset = 0;
    graph->cursPos = 0;
    Xg3d_oncurs(graph);
    break;
#ifdef ENABLE_ROW_PLOT
  case MK_ROW:
    XgInitializeStruct(&graphinfo);
    n = graph->xaxis_layout.max - graph->xaxis_layout.min;
    data = (double *) calloc(n, sizeof(double));
    if (data == NULL)
      {
	fprintf(stderr, "Calloc Failure opening line graph\n");
	return;
      }
    for (i=0; i<n; ++i)
      {
	*(data + 2*i) = (double) i + graph->xaxis_layout.min;
	*(data + 2*i + 1) = Xg3dScale_Func(i + graph->xaxis_layout.win_min,
					   graph->cursDset, graph);
      }
    graphinfo.num_sets = 1;
    graphinfo.dataset[0].darray = data;
    graphinfo.dataset[0].num_points = n;
    graphinfo.xaxis_layout.increment = XgLast5;
    strcpy(graphinfo.dataset[0].filename, "row data from Xg3d");
    sprintf(title_buf, "Row Profile from %s", graph->dataset[0].filename);
    strcpy(graphinfo.title_text, title_buf);
    sprintf(title_buf, "Row number %d", graph->cursDset);
    strcpy(graphinfo.subtitle_text, title_buf);
    row_graph = XtAppCreateShell("Row Profile", XgClass,
			     applicationShellWidgetClass,
			     XtDisplay(graph->app_shell), NULL, 0);
    graphinfo.app_shell = row_graph;
    XgCreateGraph(&graphinfo);
    break;
#endif
  default:
/*    printf("The keysym is %d\n", keysym);*/
    break;
  }
}
/**************************/
double Xg3dScale_Func(x, y, graph)
     int
       x,
       y;
     Xg3dGraphInfo
       *graph;
{
  double
    rslt;
switch (graph->dataset[0].s_func)
  {
  case Xg3dLinear:
    rslt = (double) *(graph->dataset[0].darray + y*graph->dataset[0].nx + x);
    break;
  case Xg3dLog:
    if (*(graph->dataset[0].darray + y*graph->dataset[0].nx + x) < 1)
      rslt = 0.;
    else
      rslt = (double) log((double) *(graph->dataset[0].darray
				  + y*graph->dataset[0].nx + x));
    break;
  default:
    rslt = (double) *(graph->dataset[0].darray + y*graph->dataset[0].nx + x);
    break;
  }
return(rslt);
}
