/************************************************************************
*
*                             XgCursor.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*        Cursor Functions
*
************************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

#define HOME    65360
#define LEFTA   65361
#define UPA     65362
#define RIGHTA  65363
#define DOWNA   65364

/****************************/
void Xg_CursorToggle(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
if (graph->cursor_status == XgCursOff)
  Xg_initcurs(graph);
else if (graph->cursor_status == XgCursOn)
  Xg_delcurs(graph);
else
  fprintf(stderr, "ERROR: Bogus value of cursor_status in Xg_CursorToggle\n");
}

/****************************/
void Xg_initcurs(graph)
     XgGraphInfo
       *graph;
{
  int
    i,
    left_right,
    scaling;
  double
    xr,
    yr,
    xmin,
    xmax,
    ymin,
    ymax;
  DEV_COORD
    x,
    y;
  XWindowAttributes
    attrib;

if (graph->cursor_status == XgCursOff) /* this is a turn on, not a redraw */
  {
    graph->cursDset = 0;
    graph->cursPos = 0;
  }

xmin = graph->xaxis_layout.sclmin;
xmax = graph->xaxis_layout.sclmax;
if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    xmin = pow(10., xmin);
    xmax = pow(10., xmax);
  }

left_right = graph->dataset[graph->cursDset].left_right;
if (left_right == XgLeft)
  {
    scaling = graph->yaxis_layout.scaling;
    ymin = graph->yaxis_layout.sclmin;
    ymax = graph->yaxis_layout.sclmax;
  }
else if (left_right == XgRight)
  {
    scaling = graph->y_rt_axis_layout.scaling;
    ymin = graph->y_rt_axis_layout.sclmin;
    ymax = graph->y_rt_axis_layout.sclmax;
  }

if (scaling == XgLogarithm)
  {
    ymin = pow(10., ymin);
    ymax = pow(10., ymax);
  }
if (scaling == XgProbability)
  {
    if (ymin < 0)
      ymin = .5 - erf(-ymin) / 2.;
    else
      ymin = erf(ymin) / 2. + .5;
    if (ymax < 0)
      ymax = .5 - erf(-ymax) / 2.;
    else
      ymax = erf(ymax) / 2. + .5;
  }

if (graph->cursor_status == XgCursOff) /* this is a turn on, not a redraw */
  {
    while ( (graph->cursPos < graph->dataset[0].num_points) &&
	   ( (*(graph->dataset[0].darray + 2*graph->cursPos) < xmin)
	    || (*(graph->dataset[0].darray + 2*graph->cursPos + 1) < ymin)
	    || (*(graph->dataset[0].darray + 2*graph->cursPos) > xmax)
	    || (*(graph->dataset[0].darray + 2*graph->cursPos + 1) > ymax)))
      {
	++graph->cursPos;
      }

    /***** Create Cursor Pixmap *****/
    graph->cursWidth = 10;
    graph->cursHeight = 10;
    XGetWindowAttributes(graph->display, XtWindow(graph->draw_area),
			 &attrib);
    graph->savePixmap = XCreatePixmap(graph->display,
				      XtWindow(graph->draw_area),
				      graph->cursWidth, graph->cursHeight,
				      attrib.depth);
    graph->cursPixmap = XCreatePixmap(graph->display,
				      XtWindow(graph->draw_area),
				      graph->cursWidth, graph->cursHeight,
				      attrib.depth);
    XSetForeground(graph->display, graph->theGC, graph->background_color);
    XFillRectangle(graph->display, graph->cursPixmap, graph->theGC,
		   0, 0, graph->cursWidth, graph->cursHeight);
    XSetForeground(graph->display, graph->theGC, graph->cursor_color);
    for (i=0; i<graph->cursHeight; ++i)
      XDrawPoint(graph->display, graph->cursPixmap, graph->theGC,
		 (graph->cursWidth-1)/2, i);
    for (i=0; i<graph->cursWidth; ++i)
      XDrawPoint(graph->display, graph->cursPixmap, graph->theGC, i,
		 (graph->cursHeight-1)/2);

    /** create cursor position dialog **/
    XtAddCallback(graph->draw_area, XmNinputCallback, Xg_processArrow, graph);
  }  /** end of cursor off, turn on proceedures **/

graph->cursor_status = XgCursOn;  /* turn cursor on */

/******* save current data point region *****/
xr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos);
yr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1);
if (graph->xaxis_layout.scaling == XgLogarithm)
  xr = log10(xr);
if (scaling == XgLogarithm)
  yr = log10(yr);
else if (scaling == XgProbability)
  yr = Xg_inverf(yr);
Xg_Mapwtod(xr, yr, &x, &y, graph, left_right);
XCopyArea(graph->display, XtWindow(graph->draw_area),
	  graph->savePixmap, graph->theGC,
	  x-(graph->cursWidth-1)/2, y-(graph->cursHeight-1)/2,
	  graph->cursWidth, graph->cursHeight, 0, 0);
Xg_oncurs(graph);
}

/***************************/
void Xg_delcurs(graph)
     XgGraphInfo
       *graph;
{
  int
    left_right,
    scaling;
  double
    xr,
    yr;
  DEV_COORD
    x,
    y;

/** remove position pop-up **/
printf("\n");
graph->cursor_status = XgCursOff;  /* turn cursor off */

/******* restore data point region ******/
xr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos);
yr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1);
if (graph->xaxis_layout.scaling == XgLogarithm)
  xr = log10(xr);

left_right = graph->dataset[graph->cursDset].left_right;
if (left_right == XgLeft)
  {
    scaling = graph->yaxis_layout.scaling;
  }
else if (left_right == XgRight)
  {
    scaling = graph->y_rt_axis_layout.scaling;
  }
if (scaling == XgLogarithm)
  yr = log10(yr);
if (scaling == XgProbability)
  yr = Xg_inverf(yr);

Xg_Mapwtod(xr, yr, &x, &y, graph, left_right);
XCopyArea(graph->display, graph->savePixmap,
	  XtWindow(graph->draw_area), graph->theGC, 0, 0, graph->cursWidth,
	  graph->cursHeight, x-(graph->cursWidth-1)/2,
	  y-(graph->cursHeight-1)/2);
XtRemoveAllCallbacks(graph->draw_area, XmNinputCallback);
XFreePixmap(graph->display, graph->savePixmap);
XFreePixmap(graph->display, graph->cursPixmap);
}

/***************************/
void Xg_oncurs(graph)
     XgGraphInfo
       *graph;
{
  int
    left_right,
    scaling;
  double
    xr,
    yr;
  DEV_COORD
    x,
    y;

/** update cursor readout **/
printf("dset = %d; point = %d; x = %lg; y = %lg                    \r",
       graph->cursDset, graph->cursPos,
       *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos),
       *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1));
fflush(stdout);

/******** save data point region ********/
xr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos);
yr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1);
if (graph->xaxis_layout.scaling == XgLogarithm)
  xr = log10(xr);

left_right = graph->dataset[graph->cursDset].left_right;
if (left_right == XgLeft)
  {
    scaling = graph->yaxis_layout.scaling;
  }
else if (left_right == XgRight)
  {
    scaling = graph->y_rt_axis_layout.scaling;
  }
if (scaling == XgLogarithm)
  yr = log10(yr);
if (scaling == XgProbability)
  yr = Xg_inverf(yr);
Xg_Mapwtod(xr, yr, &x, &y, graph, left_right);
XCopyArea(graph->display, XtWindow(graph->draw_area),
	  graph->savePixmap, graph->theGC,
	  x-(graph->cursWidth-1)/2, y-(graph->cursHeight-1)/2,
	  graph->cursWidth, graph->cursHeight, 0, 0);
XSetFunction(graph->display, graph->theGC, GXand);
XCopyArea(graph->display, graph->cursPixmap,
	  XtWindow(graph->draw_area), graph->theGC, 0, 0, graph->cursWidth,
	  graph->cursHeight, x-(graph->cursWidth-1)/2,
	  y-(graph->cursHeight-1)/2);
XSetFunction(graph->display, graph->theGC, GXcopy);
XFlush(graph->display);
}

/***************************/
void Xg_processArrow(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;
  XmDrawingAreaCallbackStruct
    *theCallback;
  XEvent
    *theEvent;
  XKeyEvent
    *keyEvent;
  KeySym
    keysym;
  DEV_COORD
    x,
    y;
  int
    left_right,
    scaling;
  double
    xr,
    yr,
    xmin,
    ymin,
    xmax,
    ymax;
  char
    KeyBuf[6];

graph = (XgGraphInfo *) client_data;
if (graph->cursor_status == XgCursOff)  /* cursor is off, ignore & return */
  return;

theCallback = (XmDrawingAreaCallbackStruct *) call_data;
theEvent = theCallback->event;
if (theEvent->type != KeyPress) /* not interested, return */
  return;
keyEvent = (XKeyEvent *) theEvent;
XLookupString(keyEvent, KeyBuf, 4, &keysym, NULL);

xmin = graph->xaxis_layout.sclmin;
xmax = graph->xaxis_layout.sclmax;
if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    xmin = pow(10., xmin);
    xmax = pow(10., xmax);
  }

left_right = graph->dataset[graph->cursDset].left_right;
if (left_right == XgLeft)
  {
    scaling = graph->yaxis_layout.scaling;
    ymin = graph->yaxis_layout.sclmin;
    ymax = graph->yaxis_layout.sclmax;
  }
else if (left_right == XgRight)
  {
    scaling = graph->y_rt_axis_layout.scaling;
    ymin = graph->y_rt_axis_layout.sclmin;
    ymax = graph->y_rt_axis_layout.sclmax;
  }
if (scaling == XgLogarithm)
  {
    ymin = pow(10., ymin);
    ymax = pow(10., ymax);
  }
if (scaling == XgProbability)
  {
    if (ymin < 0)
      ymin = .5 - erf(-ymin) / 2.;
    else
      ymin = erf(ymin) / 2. + .5;
    if (ymax < 0)
      ymax = .5 - erf(-ymax) / 2.;
    else
      ymax = erf(ymax) / 2. + .5;
  }

xr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos);
yr = *(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1);
if (graph->xaxis_layout.scaling == XgLogarithm)
  xr = log10(xr);
if (scaling == XgLogarithm)
  yr = log10(yr);
if (scaling == XgProbability)
  yr = Xg_inverf(yr);
Xg_Mapwtod(xr, yr, &x, &y, graph, left_right);

switch (keysym)
  {
  case LEFTA:
    /***** restore previous data point pixmap ******/
    XCopyArea(graph->display, graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    /******** move cursor ******/
    --graph->cursPos;
    if ((graph->cursPos < 0)
	|| (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos)
	    < xmin)
	|| (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1)
	    < ymin)
	|| (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos)
	    > xmax)
	|| (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos + 1)
	    > ymax) )
      {
	graph->cursPos = graph->dataset[graph->cursDset].num_points - 1;
	while ((graph->cursPos > 0)
	    && ((*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos)
		 > xmax)
	    || (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos +1)
		 > ymax)
	    || (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos)
		 < xmin)
	    || (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos +1)
		 < ymin) ))
	  {
	    --graph->cursPos;
	  }
      }
    Xg_oncurs(graph);
    break;
  case RIGHTA:
    XCopyArea(graph->display, graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    ++graph->cursPos;
    if ( (graph->cursPos > graph->dataset[graph->cursDset].num_points - 1)
	|| (*(graph->dataset[graph->cursDset].darray  +2*graph->cursPos)
	    > xmax)
	|| (*(graph->dataset[graph->cursDset].darray  +2*graph->cursPos + 1)
	    > ymax)
	|| (*(graph->dataset[graph->cursDset].darray  +2*graph->cursPos)
	    < xmin)
	|| (*(graph->dataset[graph->cursDset].darray  +2*graph->cursPos + 1)
	    < ymin) )
      {
	graph->cursPos = 0;
	while ((graph->cursPos < graph->dataset[graph->cursDset].num_points -1)
	     && ((*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos)
		  < xmin)
	     || (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos+1)
		  < ymin)
	     || (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos)
		  > xmax)
	     || (*(graph->dataset[graph->cursDset].darray + 2*graph->cursPos+1)
		  > ymax) ))
	  {
	    ++graph->cursPos;
	  }
      }
    Xg_oncurs(graph);
    break;
  case DOWNA:
    XCopyArea(graph->display, graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    --graph->cursDset;
    if (graph->cursDset < 0)
      graph->cursDset = graph->num_sets - 1;
    if (graph->cursPos > graph->dataset[graph->cursDset].num_points - 1)
      graph->cursPos = 0;
    Xg_oncurs(graph);
    break;
  case UPA:
    XCopyArea(graph->display, graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    ++graph->cursDset;
    if (graph->cursDset > graph->num_sets - 1)
      graph->cursDset = 0;
    if (graph->cursPos > graph->dataset[graph->cursDset].num_points - 1)
      graph->cursPos = 0;
    Xg_oncurs(graph);
    break;
  case HOME:
    XCopyArea(graph->display, graph->savePixmap,
	      XtWindow(graph->draw_area), graph->theGC, 0, 0,
	      graph->cursWidth, graph->cursHeight, x-(graph->cursWidth-1)/2,
	      y-(graph->cursHeight-1)/2);
    graph->cursDset = 0;
    graph->cursPos = 0;
    while ((graph->cursPos < graph->dataset[0].num_points-1)
	   && ((*(graph->dataset[0].darray + 2*graph->cursPos) < xmin)
	       || (*(graph->dataset[0].darray + 2*graph->cursPos + 1) < ymin)
	       || (*(graph->dataset[0].darray + 2*graph->cursPos) > xmax)
	       || (*(graph->dataset[0].darray + 2*graph->cursPos + 1) > ymax)))
      {
	++graph->cursPos;
      }
    Xg_oncurs(graph);
    break;
  default:
    break;
  }

}
