/**********************************************************************
*
*                            XgDrawData.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Plot Data Points on Graph
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/**********************************************************************/
void Xg_plotData(XgGraphInfo * graph)
{

  int
    i,
    j,
    size,
    flag,
    scaling,
    left_right,
    num_points;
  double
    x,
    y,
    x1,
    y1,
    y2,
    err,
    minx,
    maxx,
    miny,
    maxy;
  DEV_COORD
    xd,
    yd,
    xd1,
    yd1,
    dy,
    dx;
  XPoint
    polypts[3];

minx = graph->xaxis_layout.sclmin;
maxx = graph->xaxis_layout.sclmax;

/*********** draw data ***************/
for (j=0; j<graph->num_sets; ++j)
  {
    num_points = graph->dataset[j].num_points;
    size = graph->dataset[j].point_size - XgSize0;
    XSetForeground(XtDisplay(graph->draw_area), graph->theGC,
		   graph->dataset[j].description.color);
    left_right = graph->dataset[j].left_right;
    if (left_right == XgLeft)
      {
	scaling = graph->yaxis_layout.scaling;
	miny = graph->yaxis_layout.sclmin;
	maxy = graph->yaxis_layout.sclmax;
      }
    else if (left_right == XgRight)
      {
	scaling = graph->y_rt_axis_layout.scaling;
	miny = graph->y_rt_axis_layout.sclmin;
	maxy = graph->y_rt_axis_layout.sclmax;
      }
    else
      scaling = -1;

    /****** draw points ******/
    for (i=0; i<num_points; ++i)
      {
	x = *(graph->dataset[j].darray + 2*i);
	y = *(graph->dataset[j].darray + 2*i + 1);

	if (graph->xaxis_layout.scaling == XgLogarithm)
	  {
	    if (x > 0.)
	      x = log10(x);
	    else
	      x = minx - 1;
	  }

	if (scaling == XgLogarithm)
	  {
	    if (y > 0.)
	      y = log10(y);
	    else
	      y = miny - 1;
	  }
	else if (scaling == XgProbability)
	  {
	    if ((y >= 0.) && (y <= 1.))
	      y = Xg_inverf(y);
	    else
	      y = miny - 1;
	  }

	if ( ((x >= minx) &&  (x <= maxx))
	    && ((y >= miny) && (y <= maxy)) )
	  {
	    Xg_Mapwtod(x, y, &xd, &yd, graph, left_right);

	    switch (graph->dataset[j].point_type)
	      {
	      case XgPoint:
		XDrawPoint(graph->display, XtWindow(graph->draw_area),
			   graph->theGC, xd, yd);
		break;
	      case XgCircle:
		if (graph->dataset[j].filled_open == XgFilled)
		  XFillArc(graph->display, XtWindow(graph->draw_area),
			   graph->theGC, xd-size/2, yd-size/2, size, size,
			   0, 23040);
		else if (graph->dataset[j].filled_open == XgOpen)
		  XDrawArc(graph->display, XtWindow(graph->draw_area),
			   graph->theGC, xd-size/2, yd-size/2, size, size,
			   0, 23040);
		break;
	      case XgSquare:
		if (graph->dataset[j].filled_open == XgFilled)
		  XFillRectangle(graph->display, XtWindow(graph->draw_area),
				 graph->theGC, xd-size/2, yd-size/2,size,size);
		else if (graph->dataset[j].filled_open == XgOpen)
		  XDrawRectangle(graph->display, XtWindow(graph->draw_area),
				 graph->theGC, xd-size/2, yd-size/2,size,size);
		break;
	      case XgTriangle:
		if (graph->dataset[j].filled_open == XgFilled)
		  {
		    polypts[0].x = xd - size/2;
		    polypts[0].y = yd + size/2;
		    polypts[1].x = xd + size/2;
		    polypts[1].y = yd + size/2;
		    polypts[2].x = xd;
		    polypts[2].y = yd - size/2;
		    XFillPolygon(graph->display, XtWindow(graph->draw_area),
				 graph->theGC, polypts, 3, Convex,
				 CoordModeOrigin);
		  }
		else if (graph->dataset[j].filled_open == XgOpen)
		  {
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, xd-size/2, yd+size/2,
			      xd+size/2,yd+size/2);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, xd+size/2, yd+size/2,
			      xd, yd-size/2);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, xd, yd-size/2,
			      xd-size/2, yd+size/2);
		  }
		break;
	      case XgPlus:
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, xd-size/2, yd, xd+size/2, yd);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, xd, yd-size/2, xd, yd+size/2);
		break;
	      case XgEx:
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, xd-size/2, yd+size/2,
			  xd+size/2, yd-size/2);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, xd-size/2, yd-size/2,
			  xd+size/2, yd+size/2);
		break;
	      case XgBar:
		/* x, y, width, height, where x, y are the upper left corner
		   and upper left is 0, 0 for wind */
		Xg_Mapwtod(x, miny, &xd1, &yd1, graph, left_right);
		dy = yd1 - yd;
		dx = 4 * (size - 2);
		XFillRectangle(graph->display, XtWindow(graph->draw_area),
			       graph->theGC, xd-dx/2, yd, dx, dy);
		break;
	      default:
		break;
	      }
	  }  /** end of valid point check **/
      } /** end loop over points **/

    /********* draw error bars ********/
    if ( (graph->dataset[j].error_pts == XgHasErrorPts)
	&& (graph->dataset[j].error_bars == XgErrorOn) )
      {
	for (i=0; i<num_points; ++i)
	  {
	    x = *(graph->dataset[j].darray + 2*i);
	    y = *(graph->dataset[j].darray + 2*i + 1);
	    err = *(graph->dataset[j].err_array + i);
	    y1 = y + err;
	    y2 = y - err;

	    if (graph->xaxis_layout.scaling == XgLogarithm)
	      {
		if (x > 0.)
		  x = log10(x);
		else
		  x = minx - 1;
	      }
	    if (scaling == XgLogarithm)
	      {
		if (y > 0.)
		  y = log10(y);
		else
		  y = miny - 1;
	      }
	    else if (scaling == XgProbability)
	      {
		if ((y >= 0.) && (y <= 1.))
		  y = Xg_inverf(y);
		else
		  y = miny - 1;
	      }

	    if ( ((x >= minx) &&  (x <= maxx))  /** basic point is in graph **/
		&& ((y >= miny) && (y <= maxy)) )
	      {
		Xg_Mapwtod(x, y, &xd, &yd, graph, left_right);
		if (scaling == XgLogarithm)
		  {
		    if (y1 > 0.)
		      y1 = log10(y1);
		    else
		      y1 = miny - 1;
		    if (y2 > 0.)
		      y2 = log10(y2);
		    else
		      y2 = miny - 1;
		  }
		else if (scaling == XgProbability)
		  {
		    if ((y1 >= 0.) && (y1 <= 1.))
		      y1 = Xg_inverf(y1);
		    else
		      y1 = miny - 1;
		    if ((y2 >= 0.) && (y2 <= 1.))
		      y2 = Xg_inverf(y2);
		    else
		      y2 = miny - 1;
		  }

		/***** upper extension *******/
		if ((y1 >= miny) && (y1 <= maxy))
		  {
		    Xg_Mapwtod(x, y1, &xd1, &yd1, graph, left_right);
		    XDrawLine(graph->display,
			      XtWindow(graph->draw_area),
			      graph->theGC, xd, yd, xd1, yd1);
		    XDrawLine(graph->display,  /** cross bar **/
			      XtWindow(graph->draw_area),
			      graph->theGC, xd1-3, yd1, xd1+3, yd1);
		  }
		/***** lower extension *****/
		if ((y2 >= miny) && (y2 <= maxy))
		  {
		    Xg_Mapwtod(x, y2, &xd1, &yd1, graph, left_right);
		    XDrawLine(graph->display,
			      XtWindow(graph->draw_area),
			      graph->theGC, xd, yd, xd1, yd1);
		    XDrawLine(graph->display,  /** cross bar **/
			      XtWindow(graph->draw_area),
			      graph->theGC, xd1-3, yd1, xd1+3, yd1);
		  }
	      }  /** end of basic point is in graph **/
	  }  /** end of i-loop over data points **/
      }  /** end of if error_pts && error_bars **/

    /****** draw lines *******/
    if (graph->dataset[j].line_status == XgSolidLine)
      {
/*	gcvalues.line_style = LineOnOffDash;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	for (i=0; i<num_points-1; ++i)
	  {
	    x = *(graph->dataset[j].darray + 2*i);
	    y = *(graph->dataset[j].darray + 2*i + 1);
	    x1 = *(graph->dataset[j].darray + 2*(i+1));
	    y1 = *(graph->dataset[j].darray + 2*(i+1) + 1);

	    if (graph->xaxis_layout.scaling == XgLogarithm)
	      {
		if (x > 0.)
		  x = log10(x);
		else
		  x = minx - 1;
		if (x1 > 0.)
		  x1 = log10(x1);
		else
		  x1 = minx - 1;
	      }

	    if (scaling == XgLogarithm)
	      {
		if (y > 0.)
		  y = log10(y);
		else
		  y = miny - 1;
		if (y1 > 0.)
		  y1 = log10(y1);
		else
		  y1 = miny - 1;
	      }
	    else if (scaling == XgProbability)
	      {
		if ((y >= 0.) && (y <= 1.))
		  y = Xg_inverf(y);
		else
		  y = miny - 1;
		if ((y1 >= 0.) && (y1 <= 1.))
		  y1 = Xg_inverf(y1);
		else
		  y1 = miny - 1;
	      }

	    if ( ((x >= minx) &&  (x <= maxx))
		&& ((y >= miny) && (y <= maxy))
		&&  ((x1 >= minx) &&  (x1 <= maxx))
		&& ((y1 >= miny) && (y1 <= maxy)) )
	      {
		Xg_Mapwtod(x, y, &xd, &yd, graph, left_right);
		Xg_Mapwtod(x1, y1, &xd1, &yd1, graph, left_right);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, xd, yd, xd1, yd1);
	      }
	  }
/*	gcvalues.line_style = LineSolid;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
      } /** end line draw **/

  } /** end loop over data sets **/
}
