/**********************************************************************
*
*                            XgDrawGraph.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Main graph drawing routine(s)
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/DrawingA.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/*************************************************/
void Xg_createGraph(XgGraphInfo * graph)
{
  int
    nargs;
  Arg
    args[15];
  XGCValues
    gcvalues;
  unsigned long
    gcmask;

nargs = 0;
XtSetArg(args[nargs], XmNborderWidth, 0); ++nargs;
XtSetArg(args[nargs], XmNbackground, graph->background_color); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, graph->menu_bar); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
graph->draw_area = XmCreateDrawingArea(graph->form, "draw area", args, nargs);
XtManageChild(graph->draw_area);

XtRealizeWidget(graph->app_shell);
XtAddCallback(graph->draw_area, XmNexposeCallback, Xg_drawGraph, graph);
XtAddEventHandler(graph->draw_area, ButtonPressMask, FALSE, Xg_drag_start,
		  graph);
XtAddEventHandler(graph->draw_area, PointerMotionMask, FALSE, Xg_drag_update,
		  graph);
XtAddEventHandler(graph->draw_area, ButtonReleaseMask, FALSE, Xg_drag_finish,
		  graph);
graph->theGC = Xg_createGC(graph);
gcvalues.function = GXinvert;     /** instead of GXxor **/
/*gcvalues.line_style = LineSolid;*/
gcvalues.line_style = LineOnOffDash;
gcmask = GCFunction | GCLineStyle;
graph->gcxor = XCreateGC(graph->display, XtWindow(graph->draw_area),
			 gcmask, &gcvalues);

}

/************************************/
GC Xg_createGC(graph)
     XgGraphInfo
       *graph;
{
  XGCValues
    theGCValues;
  GC
    theGC;
theGC = (GC) XCreateGC(graph->display, XtWindow(graph->draw_area),
		  (unsigned long) NULL, &theGCValues);
if (theGC == NULL)
  {
    fprintf(stderr, "Error creating GC: Aborted\n");
    exit(1);
  }
else
    {
      XSetForeground(graph->display, theGC, graph->axis_color);
      XSetBackground(graph->display, theGC, graph->background_color);
      return(theGC);
    }
}

/*************************************************/
void Xg_drawGraph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  double
    x,
    y;
  DEV_COORD
    nx,
    ny,
    xd0,
    yd0,
    xd1,
    yd1;
  Arg
    args[15];
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
Xg_scale_data(graph);
graph->rubber_band = -1;

/********* set world ************/
x = (graph->xaxis_layout.sclmax - graph->xaxis_layout.sclmin) * .2;
y = (graph->yaxis_layout.sclmax - graph->yaxis_layout.sclmin) * .2;
Xg_Setworld(graph->xaxis_layout.sclmin - x, graph->yaxis_layout.sclmin - y,
	   graph->xaxis_layout.sclmax + x, graph->yaxis_layout.sclmax + y,
	   graph, XgLeft);
if (graph->use_right == TRUE)  /** set by Xg_scale_data() **/
  {
    y = (graph->y_rt_axis_layout.sclmax - graph->y_rt_axis_layout.sclmin) * .2;
    Xg_Setworld(graph->xaxis_layout.sclmin - x,
		graph->y_rt_axis_layout.sclmin - y,
		graph->xaxis_layout.sclmax + x,
		graph->y_rt_axis_layout.sclmax + y,
		graph, XgRight);
  }

/*************** get current sizes ******************/
nargs = 0;
XtSetArg(args[nargs], XmNwidth, &nx); ++nargs;
XtSetArg(args[nargs], XmNheight, &ny); ++nargs;
XtGetValues(graph->draw_area, args, nargs);
graph->nxdraw = nx;
graph->nydraw = ny;

XClearArea(graph->display, XtWindow(graph->draw_area), 0, 0, nx, ny, False);

/***************** draw graph box ****************/
XSetForeground(graph->display, graph->theGC, graph->axis_color);
Xg_Mapwtod(graph->xaxis_layout.sclmin, graph->yaxis_layout.sclmin,
	   &xd0, &yd0, graph, XgLeft);
Xg_Mapwtod(graph->xaxis_layout.sclmax, graph->yaxis_layout.sclmax,
	   &xd1, &yd1, graph, XgLeft);

XDrawRectangle(graph->display, XtWindow(graph->draw_area),
	       graph->theGC, xd0, yd1, xd1-xd0, yd0-yd1);

/************** draw graph components ****************/
Xg_drawLabels(graph);
Xg_drawAxes(graph);
Xg_drawDivMarks(graph);
Xg_plotData(graph);
Xg_drawLegend(graph);

if (graph->cursor_status == XgCursOn)   /** cursor is on, redraw it **/
  Xg_initcurs(graph);

XFlush(XtDisplay(graph->draw_area));
}

/*************************************************/
void Xg_drag_start(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    x,
    y,
    x0,
    y0,
    dx,
    dy;
  double
    x1,
    y1;
  XEvent
    *mouse_ev;
  XgGraphInfo
    *graph;
  static XgWidStruct
    wid_array;

graph = (XgGraphInfo *) client_data;
mouse_ev = (XEvent *) call_data;

x = mouse_ev->xmotion.x;
y = mouse_ev->xmotion.y;
graph->rubber_band = -1;

/********** x axis description *********/
x0 = graph->x_axis_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->x_axis_desc.coord.y * graph->nydraw / 100.;
dx = graph->x_axis_desc.dimen.x;
dy = graph->x_axis_desc.dimen.y;
x0 = x0 - dx / 2;
y0 = y0 - dy;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgXaxisLabel;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgXaxisLabel;
	wid_array.graph = (caddr_t) graph;
	Xg_do_edit_titles(NULL, &wid_array, NULL);
      }
  }

/********** Left y axis description *********/
x0 = graph->y_axis_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->y_axis_desc.coord.y * graph->nydraw / 100.;
dx = graph->y_axis_desc.dimen.x;
dy = graph->y_axis_desc.dimen.y;
y0 = y0 - dy / 2;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgYLaxisLabel;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgYLaxisLabel;
	wid_array.graph = (caddr_t) graph;
	Xg_do_edit_titles(NULL, &wid_array, NULL);
      }
  }

/********** Right y axis description *********/
x0 = graph->y_rt_axis_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->y_rt_axis_desc.coord.y * graph->nydraw / 100.;
dx = graph->y_rt_axis_desc.dimen.x;
dy = graph->y_rt_axis_desc.dimen.y;
y0 = y0 - dy / 2;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgYRaxisLabel;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgYRaxisLabel;
	wid_array.graph = (caddr_t) graph;
	Xg_do_edit_titles(NULL, &wid_array, NULL);
      }
  }

/******** Title ********/
x0 = graph->title_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->title_desc.coord.y * graph->nydraw / 100.;
dx = graph->title_desc.dimen.x;
dy = graph->title_desc.dimen.y;
x0 = x0 - dx / 2;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgTitle;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgTitle;
	wid_array.graph = (caddr_t) graph;
	Xg_do_edit_titles(NULL, &wid_array, NULL);
      }
  }

/********* subtitle *********/
x0 = graph->subtitle_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->subtitle_desc.coord.y * graph->nydraw / 100.;
dx = graph->subtitle_desc.dimen.x;
dy = graph->subtitle_desc.dimen.y;
x0 = x0 - dx / 2;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgSubtitle;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgSubtitle;
	wid_array.graph = (caddr_t) graph;
	Xg_do_edit_titles(NULL, &wid_array, NULL);
      }
  }

/********* x scaling info *********/
x0 = graph->xaxis_layout.scale_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->xaxis_layout.scale_desc.coord.y * graph->nydraw / 100.;
dx = graph->xaxis_layout.scale_desc.dimen.x;
dy = graph->xaxis_layout.scale_desc.dimen.y;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgXScale;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgXaxis;
	wid_array.graph = (caddr_t) graph;
	Xg_configure_axes(NULL, &wid_array, NULL);
      }
  }

/********* Left y scaling info *********/
x0 = graph->yaxis_layout.scale_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->yaxis_layout.scale_desc.coord.y * graph->nydraw / 100.;
dx = graph->yaxis_layout.scale_desc.dimen.x;
dy = graph->yaxis_layout.scale_desc.dimen.y;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgYLScale;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgYLaxis;
	wid_array.graph = (caddr_t) graph;
	Xg_configure_axes(NULL, &wid_array, NULL);
      }
  }

/********* Right y scaling info *********/
x0 = graph->y_rt_axis_layout.scale_desc.coord.x * graph->nxdraw / 100.;
y0 = graph->y_rt_axis_layout.scale_desc.coord.y * graph->nydraw / 100.;
dx = graph->y_rt_axis_layout.scale_desc.dimen.x;
dy = graph->y_rt_axis_layout.scale_desc.dimen.y;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgYRScale;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgYRaxis;
	wid_array.graph = (caddr_t) graph;
	Xg_configure_axes(NULL, &wid_array, NULL);
      }
  }

/********** legend *****/
x0 = graph->legend.description.coord.x * graph->nxdraw / 100.;
y0 = graph->legend.description.coord.y * graph->nydraw / 100.;
dx = graph->legend.description.dimen.x;
dy = graph->legend.description.dimen.y;
if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
  {
    if (mouse_ev->xbutton.button == Button1)
      {
	graph->rubber_band = XgLegend;
	graph->rbr_xoff = x - x0;   /** offset between cursor & box corner **/
	graph->rbr_yoff = y - y0;
	graph->rbr_dx = dx;
	graph->rbr_dy = dy;
      }
    else if (mouse_ev->xbutton.button == Button3)
      {
	wid_array.client = (caddr_t) XgLegend;
	wid_array.graph = (caddr_t) graph;
	Xg_do_edit_titles(NULL, &wid_array, NULL);
      }
  }

/********** dataset labels ********/
for (i=0; i<graph->num_sets; ++i)
  {
    x0 = graph->dataset[i].description.coord.x * graph->nxdraw / 100.;
    y0 = graph->dataset[i].description.coord.y * graph->nydraw / 100.;
    dx = graph->dataset[i].description.dimen.x;
    dy = graph->dataset[i].description.dimen.y;
    if ((x0 <= x) && (y0 <= y) && (x <= x0+dx) && (y <= y0+dy))
      {
	if (mouse_ev->xbutton.button == Button1)
	  {
	    graph->rubber_band = XgData;
	    graph->ftype = i;         /** remember which data set **/
	    graph->rbr_xoff = x - x0; /** offset between cursor & box corner */
	    graph->rbr_yoff = y - y0;
	    graph->rbr_dx = dx;
	    graph->rbr_dy = dy;
	  }
      }
  }

if (graph->rubber_band > 0)
  {
    graph->rbr_xp0 = x;
    graph->rbr_yp0 = y;
    XDrawRectangle(graph->display, XtWindow(graph->draw_area), graph->gcxor,
		   graph->rbr_xp0 - graph->rbr_xoff,
		   graph->rbr_yp0 - graph->rbr_yoff,
		   graph->rbr_dx, graph->rbr_dy);
  }
}

/*************************************************/
void Xg_drag_update(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    x,
    y;
  XEvent
    *mouse_ev;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
mouse_ev = (XEvent *) call_data;

if (graph->rubber_band < 0)
  return;
x = mouse_ev->xmotion.x;
y = mouse_ev->xmotion.y;

XDrawRectangle(graph->display, XtWindow(graph->draw_area), graph->gcxor,
	       graph->rbr_xp0 - graph->rbr_xoff,
	       graph->rbr_yp0 - graph->rbr_yoff,
	       graph->rbr_dx, graph->rbr_dy);
graph->rbr_xp0 = x;
graph->rbr_yp0 = y;
XDrawRectangle(graph->display, XtWindow(graph->draw_area), graph->gcxor,
	       graph->rbr_xp0 - graph->rbr_xoff,
	       graph->rbr_yp0 - graph->rbr_yoff,
	       graph->rbr_dx, graph->rbr_dy);
}

/*************************************************/
void Xg_drag_finish(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    x,
    y,
    x0,
    y0;
  XEvent
    *mouse_ev;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
mouse_ev = (XEvent *) call_data;

if (graph->rubber_band < 0)
  return;
x = mouse_ev->xmotion.x;
y = mouse_ev->xmotion.y;

XDrawRectangle(graph->display, XtWindow(graph->draw_area), graph->gcxor,
	       graph->rbr_xp0 - graph->rbr_xoff,
	       graph->rbr_yp0 - graph->rbr_yoff,
	       graph->rbr_dx, graph->rbr_dy);

switch (graph->rubber_band)
  {
  case XgXaxisLabel:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    x0 = x0 + graph->rbr_dx / 2;
    y0 = y0 + graph->rbr_dy;
    if (x0 < graph->rbr_dx/2)
      x0 = graph->rbr_dx/2;
    if (y0 < graph->rbr_dy)
      y0 = graph->rbr_dy;
    if (x0 > graph->nxdraw - graph->rbr_dx/2)
      x0 = graph->nxdraw - graph->rbr_dx/2;
    if (y0 > graph->nydraw)
      y0 = graph->nydraw;
    graph->x_axis_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->x_axis_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgYLaxisLabel:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    y0 = y0 + graph->rbr_dy / 2;
    if (x0 < 0)
      x0 = 0;
    if (y0 < graph->rbr_dy/2)
      y0 = graph->rbr_dy/2;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy/2)
      y0 = graph->nydraw - graph->rbr_dy/2;
    graph->y_axis_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->y_axis_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgYRaxisLabel:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    y0 = y0 + graph->rbr_dy / 2;
    if (x0 < 0)
      x0 = 0;
    if (y0 < graph->rbr_dy/2)
      y0 = graph->rbr_dy/2;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy/2)
      y0 = graph->nydraw - graph->rbr_dy/2;
    graph->y_rt_axis_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->y_rt_axis_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgTitle:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    x0 = x0 + graph->rbr_dx / 2;
    if (x0 < graph->rbr_dx/2)
      x0 = graph->rbr_dx/2;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx/2)
      x0 = graph->nxdraw - graph->rbr_dx/2;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->title_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->title_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgSubtitle:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    x0 = x0 + graph->rbr_dx / 2;
    if (x0 < graph->rbr_dx/2)
      x0 = graph->rbr_dx/2;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx/2)
      x0 = graph->nxdraw - graph->rbr_dx/2;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->subtitle_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->subtitle_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgXScale:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    if (x0 < 0)
      x0 = 0;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->xaxis_layout.scale_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->xaxis_layout.scale_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgYLScale:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    if (x0 < 0)
      x0 = 0;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->yaxis_layout.scale_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->yaxis_layout.scale_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgYRScale:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    if (x0 < 0)
      x0 = 0;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->y_rt_axis_layout.scale_desc.coord.x  = x0 * 100. / graph->nxdraw;
    graph->y_rt_axis_layout.scale_desc.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgLegend:
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    if (x0 < 0)
      x0 = 0;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->legend.description.coord.x  = x0 * 100. / graph->nxdraw;
    graph->legend.description.coord.y  = y0 * 100. / graph->nydraw;
    break;

  case XgData:
    i = graph->ftype;
    x0 = x - graph->rbr_xoff;
    y0 = y - graph->rbr_yoff;
    if (x0 < 0)
      x0 = 0;
    if (y0 < 0)
      y0 = 0;
    if (x0 > graph->nxdraw - graph->rbr_dx)
      x0 = graph->nxdraw - graph->rbr_dx;
    if (y0 > graph->nydraw - graph->rbr_dy)
      y0 = graph->nydraw - graph->rbr_dy;
    graph->dataset[i].description.coord.x  = x0 * 100. / graph->nxdraw;
    graph->dataset[i].description.coord.y  = y0 * 100. / graph->nydraw;
    break;

  default:
    break;
  }

graph->rubber_band = -1;
Xg_drawGraph(NULL, graph, NULL);
}
