/**********************************************************************
*
*                            XgLdSvGr.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Routines to save and load graph description files
*
***********************************************************************
*/
#ifdef sgi
#define USE_DIRENT
#endif
#ifdef __hpux
#define USE_DIRENT
#endif

#include "stdio.h"
#include "string.h"

/* either use the next line and then skip 2, or under strict system V,
   skip the next line and use the following 2 */
#ifndef USE_DIRENT
#include "sys/dir.h"
#else
#include "sys/types.h"
#include "dirent.h"
#endif

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/FileSB.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/************ Load or Save a graph: File Selection Box ***************/
void Xg_load_save_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Widget
    file_select;
  int
    nargs;
  Arg
    args[10];
  char
    title[80];
  XmString
    stitle,
    dirmask;
  XgWidStruct
    *wid_addr;
  XgGraphInfo
    *graph;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;
dirmask = XmStringCreateLtoR("*.grf", XgCharset);

if ((int) wid_addr->client == XgLoad)
  {
    strcpy(title, "Xg: Load Graph");
  }
else if ((int) wid_addr->client == XgSave)
  {
    strcpy(title, "Xg: Save Graph");
  }
else
  {
    fprintf(stderr, "Bad parameter in Xg_load_save_graph. Aborted!\n");
    exit(1);
  }

nargs = 0;
stitle = XmStringCreateLtoR(title, XgCharset);
XtSetArg(args[nargs], XmNdialogTitle, stitle); ++nargs;
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
XtSetArg(args[nargs], XmNdirMask, dirmask); ++nargs;
file_select = XmCreateFileSelectionDialog(graph->app_shell,
		title, args, nargs);

if ((int) wid_addr->client == XgLoad)
  XtAddCallback(file_select, XmNokCallback, Xg_load_graph, graph);
else if ((int) wid_addr->client == XgSave)
  XtAddCallback(file_select, XmNokCallback, Xg_save_graph, graph);
XtAddCallback(file_select, XmNcancelCallback, Xg_cancel_CB, NULL);
XtAddCallback(file_select, (String) XmNhelpCallback,
	      (XtCallbackProc) Xg_help_sys, (caddr_t) XgHELP_LOAD_SAVE);

XtManageChild(file_select);
XtFree(dirmask);
XtFree(stitle);
}

/**************************************/
void Xg_load_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i,
    val;
  long int
    forecolor,
    backcolor;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  FILE
    *fp;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, XgCharset, &buf);
XtFree(dialog_rslt);

fp = fopen(buf, "r");
if (fp == NULL)
  {
    XtFree(buf);
    Xg_error_sys(XgERR_NO_FILE, graph);
    return;
  }
XtDestroyWidget(w);

if (graph->cursor_status == XgCursOn)
  Xg_delcurs(graph);

/******* free old stuff ******/
for (i=0; i<XgNSETS; ++i)
  {
    if (graph->dataset[i].darray != NULL)
      free(graph->dataset[i].darray);
    if (graph->dataset[i].err_array != NULL)
      free(graph->dataset[i].err_array);
    free(graph->dataset[i].description.font_info.font_struct);
  }
free(graph->x_axis_desc.font_info.font_struct);
free(graph->y_axis_desc.font_info.font_struct);
free(graph->y_rt_axis_desc.font_info.font_struct);
free(graph->title_desc.font_info.font_struct);
free(graph->subtitle_desc.font_info.font_struct);
free(graph->xaxis_layout.scale_desc.font_info.font_struct);
free(graph->yaxis_layout.scale_desc.font_info.font_struct);
free(graph->y_rt_axis_layout.scale_desc.font_info.font_struct);
free(graph->legend.description.font_info.font_struct);

Xg_initializeStruct(graph);
XgReadGrf(fp, graph);

fclose(fp);
XtFree(buf);
Xg_finishInitialize(graph);
Xg_createGraph(graph);
return;
}

/******************/
void XgReadGrf(FILE * fp, XgGraphInfo * graph)
{
  FILE
    *fp_data;
  int
    i,
    j,
    num,
    val;
  char
    input_text[200];

/*** read in graph setup values ***/
while (Xg_Getline(fp, input_text) != EOF)
  {
    strcat(input_text, "\n");

    if (!strncmp(input_text, "num_sets: ", 10))
      sscanf(&input_text[10], "%d", &graph->num_sets);

    if (!strncmp(input_text, "x-axis text: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->x_axis_desc.text);
    if (!strncmp(input_text, "x-axis font: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->x_axis_desc.font_info.name);
    if (!strncmp(input_text, "x-axis text location x: ", 24))
      sscanf(&input_text[24], "%le", &graph->x_axis_desc.coord.x);
    if (!strncmp(input_text, "x-axis text location y: ", 24))
      sscanf(&input_text[24], "%le", &graph->x_axis_desc.coord.y);

    if (!strncmp(input_text, "y-axis text: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->y_axis_desc.text);
    if (!strncmp(input_text, "y-axis font: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->y_axis_desc.font_info.name);
    if (!strncmp(input_text, "y-axis text location x: ", 24))
      sscanf(&input_text[24], "%le", &graph->y_axis_desc.coord.x);
    if (!strncmp(input_text, "y-axis text location y: ", 24))
      sscanf(&input_text[24], "%le", &graph->y_axis_desc.coord.y);

    if (!strncmp(input_text, "y-rt-axis text: ", 16))
      sscanf(&input_text[16], "%[^\n]", graph->y_rt_axis_desc.text);
    if (!strncmp(input_text, "y-rt-axis font: ", 16))
      sscanf(&input_text[16], "%[^\n]", graph->y_rt_axis_desc.font_info.name);
    if (!strncmp(input_text, "y-rt-axis text location x: ", 27))
      sscanf(&input_text[27], "%le", &graph->y_rt_axis_desc.coord.x);
    if (!strncmp(input_text, "y-rt-axis text location y: ", 27))
      sscanf(&input_text[27], "%le", &graph->y_rt_axis_desc.coord.y);

    if (!strncmp(input_text, "title text: ", 12))
      sscanf(&input_text[12], "%[^\n]", graph->title_desc.text);
    if (!strncmp(input_text, "title font: ", 12))
      sscanf(&input_text[12], "%[^\n]", graph->title_desc.font_info.name);
    if (!strncmp(input_text, "title text location x: ", 23))
      sscanf(&input_text[23], "%le", &graph->title_desc.coord.x);
    if (!strncmp(input_text, "title text location y: ", 23))
      sscanf(&input_text[23], "%le", &graph->title_desc.coord.y);

    if (!strncmp(input_text, "subtitle text: ", 15))
      sscanf(&input_text[15], "%[^\n]", graph->subtitle_desc.text);
    if (!strncmp(input_text, "subtitle font: ", 15))
      sscanf(&input_text[15], "%[^\n]", graph->subtitle_desc.font_info.name);
    if (!strncmp(input_text, "subtitle text location x: ", 26))
      sscanf(&input_text[26], "%le", &graph->subtitle_desc.coord.x);
    if (!strncmp(input_text, "subtitle text location y: ", 26))
      sscanf(&input_text[26], "%le", &graph->subtitle_desc.coord.y);

    if (!strncmp(input_text, "legend use: ", 12))
      sscanf(&input_text[12], "%d", &graph->legend.use_legend);
    if (!strncmp(input_text, "legend text: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->legend.description.text);
    if (!strncmp(input_text, "legend font: ", 13))
      sscanf(&input_text[13], "%[^\n]",
	     graph->legend.description.font_info.name);
    if (!strncmp(input_text, "legend location x: ", 19))
      sscanf(&input_text[19], "%le", &graph->legend.description.coord.x);
    if (!strncmp(input_text, "legend location y: ", 19))
      sscanf(&input_text[19], "%le", &graph->legend.description.coord.y);

    if (!strncmp(input_text, "x-axis min: ", 12))
      sscanf(&input_text[12], "%le", &graph->xaxis_layout.min);
    if (!strncmp(input_text, "x-axis max: ", 12))
      sscanf(&input_text[12], "%le", &graph->xaxis_layout.max);
    if (!strncmp(input_text, "x-axis scale location x: ", 25))
      sscanf(&input_text[25], "%le", &graph->xaxis_layout.scale_desc.coord.x);
    if (!strncmp(input_text, "x-axis scale location y: ", 25))
      sscanf(&input_text[25], "%le", &graph->xaxis_layout.scale_desc.coord.y);
    if (!strncmp(input_text, "x-axis increment digit: ", 24))
      sscanf(&input_text[24], "%d", &graph->xaxis_layout.increment);
    if (!strncmp(input_text, "x-axis significant figures: ", 28))
      sscanf(&input_text[28], "%d", &graph->xaxis_layout.nsig_fig);
    if (!strncmp(input_text, "x-axis div marks: ", 18))
      sscanf(&input_text[18], "%d", &graph->xaxis_layout.div_marks);
    if (!strncmp(input_text, "x-axis scaling: ", 16))
      sscanf(&input_text[16], "%d", &graph->xaxis_layout.scaling);
    if (!strncmp(input_text, "x-axis labels: ", 15))
      sscanf(&input_text[15], "%d", &graph->xaxis_layout.label_type);
    if (!strncmp(input_text, "x-axis user_set_flag: ", 22))
      {
	sscanf(&input_text[22], "%d", &val);
	graph->xaxis_layout.user_set_flag = (char) val;
      }

    if (!strncmp(input_text, "y-axis min: ", 12))
      sscanf(&input_text[12], "%le", &graph->yaxis_layout.min);
    if (!strncmp(input_text, "y-axis max: ", 12))
      sscanf(&input_text[12], "%le", &graph->yaxis_layout.max);
    if (!strncmp(input_text, "y-axis scale location x: ", 25))
      sscanf(&input_text[25], "%le", &graph->yaxis_layout.scale_desc.coord.x);
    if (!strncmp(input_text, "y-axis scale location y: ", 25))
      sscanf(&input_text[25], "%le", &graph->yaxis_layout.scale_desc.coord.y);
    if (!strncmp(input_text, "y-axis increment digit: ", 24))
      sscanf(&input_text[24], "%d", &graph->yaxis_layout.increment);
    if (!strncmp(input_text, "y-axis significant figures: ", 28))
      sscanf(&input_text[28], "%d", &graph->yaxis_layout.nsig_fig);
    if (!strncmp(input_text, "y-axis div marks: ", 18))
      sscanf(&input_text[18], "%d", &graph->yaxis_layout.div_marks);
    if (!strncmp(input_text, "y-axis scaling: ", 16))
      sscanf(&input_text[16], "%d", &graph->yaxis_layout.scaling);
    if (!strncmp(input_text, "y-axis labels: ", 15))
      sscanf(&input_text[15], "%d", &graph->yaxis_layout.label_type);
    if (!strncmp(input_text, "y-axis user_set_flag: ", 22))
      {
	sscanf(&input_text[22], "%d", &val);
	graph->yaxis_layout.user_set_flag = (char) val;
      }

    if (!strncmp(input_text, "y-rt-axis min: ", 15))
      sscanf(&input_text[15], "%le", &graph->y_rt_axis_layout.min);
    if (!strncmp(input_text, "y-rt-axis max: ", 15))
      sscanf(&input_text[15], "%le", &graph->y_rt_axis_layout.max);
    if (!strncmp(input_text, "y-rt-axis scale location x: ", 28))
      sscanf(&input_text[28], "%le",
	     &graph->y_rt_axis_layout.scale_desc.coord.x);
    if (!strncmp(input_text, "y-rt-axis scale location y: ", 28))
      sscanf(&input_text[28], "%le",
	     &graph->y_rt_axis_layout.scale_desc.coord.y);
    if (!strncmp(input_text, "y-rt-axis increment digit: ", 27))
      sscanf(&input_text[27], "%d", &graph->y_rt_axis_layout.increment);
    if (!strncmp(input_text, "y-rt-axis significant figures: ", 31))
      sscanf(&input_text[31], "%d", &graph->y_rt_axis_layout.nsig_fig);
    if (!strncmp(input_text, "y-rt-axis div marks: ", 21))
      sscanf(&input_text[21], "%d", &graph->y_rt_axis_layout.div_marks);
    if (!strncmp(input_text, "y-rt-axis scaling: ", 19))
      sscanf(&input_text[19], "%d", &graph->y_rt_axis_layout.scaling);
    if (!strncmp(input_text, "y-rt-axis labels: ", 18))
      sscanf(&input_text[18], "%d", &graph->y_rt_axis_layout.label_type);
    if (!strncmp(input_text, "y-rt-axis user_set_flag: ", 25))
      {
	sscanf(&input_text[25], "%d", &val);
	graph->y_rt_axis_layout.user_set_flag = (char) val;
      }

    if (!strncmp(input_text, "dataset ", 8))
      {
	sscanf(&input_text[8], "%d", &val);
	if (val < 10) /* set offset for rest of string */
	  i = 10;
	else
	  i = 11;
	if (!strncmp(&input_text[i], "filename: ", 10))
	  sscanf(&input_text[i+10], "%s", graph->dataset[val].filename);
	if (!strncmp(&input_text[i], "points: ", 8))
	  sscanf(&input_text[i+8], "%d", &graph->dataset[val].num_points);
	if (!strncmp(&input_text[i], "point type: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].point_type);
	if (!strncmp(&input_text[i], "point size: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].point_size);
	if (!strncmp(&input_text[i], "fill/open: ", 11))
	  sscanf(&input_text[i+11], "%d", &graph->dataset[val].filled_open);
	if (!strncmp(&input_text[i], "line style: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].line_status);
	if (!strncmp(&input_text[i], "file type: ", 11))
	  sscanf(&input_text[i+11], "%d", &graph->dataset[val].file_type);
	if (!strncmp(&input_text[i], "left/right: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].left_right);
	if (!strncmp(&input_text[i], "error data: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].error_pts);
	if (!strncmp(&input_text[i], "error bars: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].error_bars);
	if (!strncmp(&input_text[i], "in legend: ", 11))
	  sscanf(&input_text[i+11], "%d", &graph->dataset[val].in_legend);
	if (!strncmp(&input_text[i], "legend text: ", 13))
	  sscanf(&input_text[i+13], "%[^\n]", graph->dataset[val].legend_text);
	if (!strncmp(&input_text[i], "description: ", 13))
	  sscanf(&input_text[i+13], "%[^\n]",
		 graph->dataset[val].description.text);
	if (!strncmp(&input_text[i], "location x: ", 12))
	  sscanf(&input_text[i+12], "%le",
		 &graph->dataset[val].description.coord.x);
	if (!strncmp(&input_text[i], "location y: ", 12))
	  sscanf(&input_text[i+12], "%le",
		 &graph->dataset[val].description.coord.y);
	if (!strncmp(&input_text[i], "font: ", 6))
	  sscanf(&input_text[i+6], "%[^\n]",
		 graph->dataset[val].description.font_info.name);
      }
  }

/*** finish up & create graph ***/
for (i=0; i<graph->num_sets; ++i)
  {
    fp_data = fopen(graph->dataset[i].filename, "r");
    if (fp_data == NULL)
      {
	fprintf(stderr, "selected bogus file: %d: %s!! Aborted\n",
		i, graph->dataset[i].filename);
	exit(0);
      }

    Xg_read_datafile(fp_data, graph, i);

    fclose(fp_data);
  }
}

/**************************************/
void Xg_save_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i,
    val;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  FILE
    *fp;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, XgCharset, &buf);
XtFree(dialog_rslt);

for (i=0; i<graph->num_sets; ++i)
  {
    if (graph->dataset[i].file_type == Xg_Not_File)
      {
	XtFree(buf);
	Xg_error_sys(XgERR_DATA_NOT_FILE, graph);
	return;
      }
  }

fp = fopen(buf, "w");
if (fp == NULL)
  {
    XtFree(buf);
    Xg_error_sys(XgERR_NO_FILE, graph);
    return;
  }
fprintf(fp, "graph name: %s\n", buf);
XtFree(buf);
XtDestroyWidget(w);

fprintf(fp, "num_sets: %d\n", graph->num_sets);

fprintf(fp, "x-axis text: %s\n", graph->x_axis_desc.text);
fprintf(fp, "x-axis font: %s\n", graph->x_axis_desc.font_info.name);
fprintf(fp, "x-axis text location x: %lg\n", graph->x_axis_desc.coord.x);
fprintf(fp, "x-axis text location y: %lg\n", graph->x_axis_desc.coord.y);

fprintf(fp, "y-axis text: %s\n", graph->y_axis_desc.text);
fprintf(fp, "y-axis font: %s\n", graph->y_axis_desc.font_info.name);
fprintf(fp, "y-axis text location x: %lg\n", graph->y_axis_desc.coord.x);
fprintf(fp, "y-axis text location y: %lg\n", graph->y_axis_desc.coord.y);

fprintf(fp, "y-rt-axis text: %s\n", graph->y_rt_axis_desc.text);
fprintf(fp, "y-rt-axis font: %s\n", graph->y_rt_axis_desc.font_info.name);
fprintf(fp, "y-rt-axis text location x: %lg\n", graph->y_rt_axis_desc.coord.x);
fprintf(fp, "y-rt-axis text location y: %lg\n", graph->y_rt_axis_desc.coord.y);

fprintf(fp, "title text: %s\n", graph->title_desc.text);
fprintf(fp, "title font: %s\n", graph->title_desc.font_info.name);
fprintf(fp, "title text location x: %lg\n", graph->title_desc.coord.x);
fprintf(fp, "title text location y: %lg\n", graph->title_desc.coord.y);

fprintf(fp, "subtitle text: %s\n", graph->subtitle_desc.text);
fprintf(fp, "subtitle font: %s\n", graph->subtitle_desc.font_info.name);
fprintf(fp, "subtitle text location x: %lg\n", graph->subtitle_desc.coord.x);
fprintf(fp, "subtitle text location y: %lg\n", graph->subtitle_desc.coord.y);

fprintf(fp, "legend use: %d\n", graph->legend.use_legend);
fprintf(fp, "legend text: %s\n", graph->legend.description.text);
fprintf(fp, "legend font: %s\n", graph->legend.description.font_info.name);
fprintf(fp, "legend location x: %lg\n", graph->legend.description.coord.x);
fprintf(fp, "legend location y: %lg\n", graph->legend.description.coord.y);

fprintf(fp, "x-axis min: %lg\n", graph->xaxis_layout.min);
fprintf(fp, "x-axis max: %lg\n", graph->xaxis_layout.max);
fprintf(fp, "x-axis scale location x: %lg\n",
	graph->xaxis_layout.scale_desc.coord.x);
fprintf(fp, "x-axis scale location y: %lg\n",
	graph->xaxis_layout.scale_desc.coord.y);
fprintf(fp, "x-axis increment digit: %d\n", graph->xaxis_layout.increment);
fprintf(fp, "x-axis significant figures: %d\n", graph->xaxis_layout.nsig_fig);
fprintf(fp, "x-axis div marks: %d\n", graph->xaxis_layout.div_marks);
fprintf(fp, "x-axis scaling: %d\n", graph->xaxis_layout.scaling);
fprintf(fp, "x-axis labels: %d\n", graph->xaxis_layout.label_type);
fprintf(fp, "x-axis user_set_flag: %d\n",
	(int) graph->xaxis_layout.user_set_flag);

fprintf(fp, "y-axis min: %lg\n", graph->yaxis_layout.min);
fprintf(fp, "y-axis max: %lg\n", graph->yaxis_layout.max);
fprintf(fp, "y-axis scale location x: %lg\n",
	graph->yaxis_layout.scale_desc.coord.x);
fprintf(fp, "y-axis scale location y: %lg\n",
	graph->yaxis_layout.scale_desc.coord.y);
fprintf(fp, "y-axis increment digit: %d\n", graph->yaxis_layout.increment);
fprintf(fp, "y-axis significant figures: %d\n", graph->yaxis_layout.nsig_fig);
fprintf(fp, "y-axis div marks: %d\n", graph->yaxis_layout.div_marks);
fprintf(fp, "y-axis scaling: %d\n", graph->yaxis_layout.scaling);
fprintf(fp, "y-axis labels: %d\n", graph->yaxis_layout.label_type);
fprintf(fp, "y-axis user_set_flag: %d\n",
	(int) graph->yaxis_layout.user_set_flag);

fprintf(fp, "y-rt-axis min: %lg\n", graph->y_rt_axis_layout.min);
fprintf(fp, "y-rt-axis max: %lg\n", graph->y_rt_axis_layout.max);
fprintf(fp, "y-rt-axis scale location x: %lg\n",
	graph->y_rt_axis_layout.scale_desc.coord.x);
fprintf(fp, "y-rt-axis scale location y: %lg\n",
	graph->y_rt_axis_layout.scale_desc.coord.y);
fprintf(fp, "y-rt-axis increment digit: %d\n",
	graph->y_rt_axis_layout.increment);
fprintf(fp, "y-rt-axis significant figures: %d\n",
	graph->y_rt_axis_layout.nsig_fig);
fprintf(fp, "y-rt-axis div marks: %d\n", graph->y_rt_axis_layout.div_marks);
fprintf(fp, "y-rt-axis scaling: %d\n", graph->y_rt_axis_layout.scaling);
fprintf(fp, "y-rt-axis labels: %d\n", graph->y_rt_axis_layout.label_type);
fprintf(fp, "y-rt-axis user_set_flag: %d\n",
	(int) graph->y_rt_axis_layout.user_set_flag);

for (i=0; i<graph->num_sets; ++i)
  {
    fprintf(fp, "\ndataset %d filename: %s\n", i, graph->dataset[i].filename);
    fprintf(fp, "dataset %d points: %d\n", i, graph->dataset[i].num_points);
    fprintf(fp, "dataset %d point type: %d\n",i,graph->dataset[i].point_type);
    fprintf(fp, "dataset %d point size: %d\n",i,graph->dataset[i].point_size);
    fprintf(fp, "dataset %d fill/open: %d\n",i,graph->dataset[i].filled_open);
    fprintf(fp, "dataset %d line style: %d\n",i,graph->dataset[i].line_status);
    fprintf(fp, "dataset %d file type: %d\n",i,graph->dataset[i].file_type);
    fprintf(fp, "dataset %d left/right: %d\n",i,graph->dataset[i].left_right);
    fprintf(fp, "dataset %d error data: %d\n",i,graph->dataset[i].error_pts);
    fprintf(fp, "dataset %d error bars: %d\n",i,graph->dataset[i].error_bars);
    fprintf(fp, "dataset %d in legend: %d\n",i,graph->dataset[i].in_legend);
    fprintf(fp,"dataset %d legend text: %s\n",i,graph->dataset[i].legend_text);
    fprintf(fp, "dataset %d description: %s\n",
	    i, graph->dataset[i].description.text);
    fprintf(fp, "dataset %d location x: %lg\n",
	    i, graph->dataset[i].description.coord.x);
    fprintf(fp, "dataset %d location y: %lg\n",
	    i, graph->dataset[i].description.coord.y);
    fprintf(fp, "dataset %d font: %s\n",
	    i, graph->dataset[i].description.font_info.name);
  }

fclose(fp);
}

