/**********************************************************************
*
*                            XgTitles.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*    axis, graph, & legend titles (text, font, & location)
*
***********************************************************************
*/

#include "stdio.h"
#include "string.h"
#ifndef Xg_NO_CTYPE
#include "ctype.h"
#endif

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/Form.h"
#include "Xm/Frame.h"
#include "Xm/SelectioB.h"
#include "Xm/PushB.h"
#include "Xm/PanedW.h"
#include "Xm/Text.h"
#include "Xm/FileSB.h"
#include "Xm/RowColumn.h"
#include "Xm/ToggleB.h"
#include "Xm/ToggleBG.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/*******************************************************************/
void Xg_do_edit_titles(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  char
    buf[80];
  Arg
    args[10];
  Widget
    label,
    text_frame,
    font_frame,
    x_frame,
    y_frame,
    text_wid,
    parent,
    button,
    form;
  XmString
    frame_label,
    title,
    defaults;

  XgWidStruct
    *wid_addr;
  XgGraphInfo
    *graph;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

switch ((int) wid_addr->client)
  {
  case XgXaxisLabel:
    strcpy(graph->temp_text, graph->x_axis_desc.text);
    strcpy(graph->temp_font, graph->x_axis_desc.font_info.name);
    graph->temp_coord.x = graph->x_axis_desc.coord.x;
    graph->temp_coord.y = graph->x_axis_desc.coord.y;
    title = XmStringCreateLtoR("Xg: X-Axis Title", XgCharset);
    break;
  case XgYLaxisLabel:
    strcpy(graph->temp_text, graph->y_axis_desc.text);
    strcpy(graph->temp_font, graph->y_axis_desc.font_info.name);
    graph->temp_coord.x = graph->y_axis_desc.coord.x;
    graph->temp_coord.y = graph->y_axis_desc.coord.y;
    title = XmStringCreateLtoR("Xg: Y-Axis Title", XgCharset);
    break;
  case XgYRaxisLabel:
    strcpy(graph->temp_text, graph->y_rt_axis_desc.text);
    strcpy(graph->temp_font, graph->y_rt_axis_desc.font_info.name);
    graph->temp_coord.x = graph->y_rt_axis_desc.coord.x;
    graph->temp_coord.y = graph->y_rt_axis_desc.coord.y;
    title = XmStringCreateLtoR("Xg: Y-Axis Title (Right)", XgCharset);
    break;
  case XgTitle:
    strcpy(graph->temp_text, graph->title_desc.text);
    strcpy(graph->temp_font, graph->title_desc.font_info.name);
    graph->temp_coord.x = graph->title_desc.coord.x;
    graph->temp_coord.y = graph->title_desc.coord.y;
    title = XmStringCreateLtoR("Xg: Graph Title", XgCharset);
    break;
  case XgSubtitle:
    strcpy(graph->temp_text, graph->subtitle_desc.text);
    strcpy(graph->temp_font, graph->subtitle_desc.font_info.name);
    graph->temp_coord.x = graph->subtitle_desc.coord.x;
    graph->temp_coord.y = graph->subtitle_desc.coord.y;
    title = XmStringCreateLtoR("Xg: Graph Subtitle", XgCharset);
    break;
  case XgLegend:
    strcpy(graph->temp_text, graph->legend.description.text);
    strcpy(graph->temp_font, graph->legend.description.font_info.name);
    graph->temp_coord.x = graph->legend.description.coord.x;
    graph->temp_coord.y = graph->legend.description.coord.y;
    title = XmStringCreateLtoR("Xg: Legend", XgCharset);
    break;
  default:
    break;
  }

/********** main popup dialog shell ************/
nargs = 0;
defaults = XmStringCreateLtoR("Defaults", XgCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNapplyLabelString, defaults); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
parent = XmCreatePromptDialog(graph->app_shell, "titles", args, nargs);
button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_APPLY_BUTTON);
XtManageChild(button);

nargs = 0;
form = XmCreateForm(parent, "form", args, nargs);
XtManageChild(form);

/** Text Title Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
text_frame = XmCreateFrame(form, "text frame", args, nargs);
XtManageChild(text_frame);

nargs = 0;
button = XmCreateForm(text_frame, "form", args, nargs);
XtManageChild(button);

nargs = 0;
frame_label = XmStringCreateLtoR("Text", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label = XmCreateLabel(button, "text Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
text_wid = XmCreateText(button, "text box", args, nargs);
XmTextSetString(text_wid, graph->temp_text);
XtManageChild(text_wid);
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_text, graph);

/** Font Name Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, text_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
font_frame = XmCreateFrame(form, "text frame", args, nargs);
XtManageChild(font_frame);

nargs = 0;
x_frame = XmCreateForm(font_frame, "font form", args, nargs);
XtManageChild(x_frame);

nargs = 0;
sprintf(buf, "Font: %s", graph->temp_font);
frame_label = XmStringCreateLtoR(buf, XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
graph->temp_widget = XmCreateLabel(x_frame, "font Label", args, nargs);
XtManageChild(graph->temp_widget);
XtFree(frame_label);

nargs = 0;
/*XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, graph->temp_widget); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;*/
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
button = XmCreatePushButton(x_frame, "Change", args, nargs);
XtManageChild(button);
XtAddCallback(button, XmNactivateCallback, Xg_choose_new_font, graph);

/** X Coord Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, font_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
x_frame = XmCreateFrame(form, "x frame", args, nargs);
XtManageChild(x_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(x_frame, "x window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("X Coordinate", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "x Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "x text", args, nargs);
sprintf(buf, "%lg", graph->temp_coord.x);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[0].client = (caddr_t) XgX_pos;
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_coord,
	      &graph->wid_array[0]);

/** Y Coord Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, x_frame); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, font_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
y_frame = XmCreateFrame(form, "y frame", args, nargs);
XtManageChild(y_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(y_frame, "y window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Y Coordinate", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "y Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "y text", args, nargs);
sprintf(buf, "%lg", graph->temp_coord.y);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[1].client = (caddr_t) XgY_pos;
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_coord,
	      &graph->wid_array[1]);

/*** Legend on/off ***/
if ((int) wid_addr->client == XgLegend)
  {
    nargs = 0;
    XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
    XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNtopWidget, x_frame); ++nargs;
    XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
    y_frame = XmCreateFrame(form, "Legend on/off", args, nargs);
    XtManageChild(y_frame);

    nargs = 0;
    if (graph->legend.use_legend == XgLegendOn)
      {
	XtSetArg(args[nargs], XmNset, True); ++nargs;
	frame_label = XmStringCreateLtoR("Legend ON/off", XgCharset);
      }
    else
      {
	XtSetArg(args[nargs], XmNset, False); ++nargs;
	frame_label = XmStringCreateLtoR("Legend on/OFF", XgCharset);
      }
    XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
    XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
    XtSetArg(args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); ++nargs;
    button = XmCreateToggleButtonGadget(y_frame, "Legend on/off button",
					args,nargs);
    XtAddCallback(button, XmNvalueChangedCallback, Xg_toggle_legend, graph);
    XtManageChild(button);
    XtFree(frame_label);

  }

/**** add main popup callbacks ****/
XtManageChild(parent);
XtAddCallback(parent, XmNcancelCallback, Xg_cancel_CB, NULL);
XtAddCallback(parent, XmNhelpCallback, Xg_help_sys, (caddr_t) XgHELP_TITLES);
XtAddCallback(parent, XmNokCallback, Xg_set_label_vals, client_data);
XtAddCallback(parent, XmNapplyCallback, Xg_titles_defaults, client_data);
XtFree(title);
XtFree(defaults);
}

/****************************/
void Xg_get_temp_text(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i;
  double
    value;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
buf = XmTextGetString(w);
strcpy(graph->temp_text, buf);
XtFree(buf);
}

/**************************************/
void Xg_choose_new_font(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Widget
    file_select;
  int
    nargs;
  Arg
    args[10];
  char
    title[80],
    dirmask[80];
  XmString
    stitle,
    sdirmask;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
sprintf(dirmask, "%s*", graph->fontDir);
sdirmask = XmStringCreateLtoR(dirmask, XgCharset);
strcpy(title, "Xg: Font Selector");

nargs = 0;
stitle = XmStringCreateLtoR(title, XgCharset);
XtSetArg(args[nargs], XmNdialogTitle, stitle); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNdirMask, sdirmask); ++nargs;
file_select = XmCreateFileSelectionDialog(graph->app_shell,
		title, args, nargs);
XtAddCallback(file_select, XmNokCallback, Xg_get_temp_font, graph);
XtAddCallback(file_select, XmNcancelCallback, Xg_cancel_CB, NULL);
XtAddCallback(file_select, (String) XmNhelpCallback,
	      (XtCallbackProc) Xg_help_sys, (caddr_t) XgHELP_FONTS);
XtManageChild(file_select);
XtFree(sdirmask);
XtFree(stitle);
}

/**************************************/
void Xg_get_temp_font(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    buf1[80],
    *buf,
    *charpt;
  int
    i,
    nargs,
    val;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  Arg
    args[10];
  XmString
    frame_label;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;

XmStringGetLtoR(dialog_rslt->value, XgCharset, &buf);
charpt = strchr(buf, '.');  /** find first '.' **/
while (charpt != NULL)      /** strip off trailing ".snf" or ".snf.Z" **/
  {
    if ( (!strncmp(charpt, ".snf", 4)) || (!strncmp(charpt, ".pcf", 4)) )
      {
	*charpt = '\0';
	charpt = NULL;
      }
    else
      charpt = strchr(charpt+1, '.');
  }
charpt = strrchr(buf, '/');  /** find last '/', to strip off path name **/
if (charpt != NULL)
  strcpy(graph->temp_font, charpt+1);
else
  strcpy(graph->temp_font, buf);

nargs = 0;
sprintf(buf1, "Font: %s", graph->temp_font);
frame_label = XmStringCreateLtoR(buf1, XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
XtSetValues(graph->temp_widget, args, nargs);

XtFree(frame_label);
XtFree(dialog_rslt);
XtFree(buf);
XtDestroyWidget(w);

return;
}

/**************************************/
void Xg_get_temp_coord(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i;
  double
    value;
  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

buf = XmTextGetString(w);
#ifndef Xg_NO_CTYPE
for (i=0; i<strlen(buf); ++i)
  if (!(isdigit(buf[i]) || isspace(buf[i]) || (buf[i] == '-')))
      return;
#endif
sscanf(buf, "%le", &value);
if ((int) wid_addr->client == XgX_pos)
  {
    graph->temp_coord.x = value;
  }
else if ((int) wid_addr->client == XgY_pos)
  {
    graph->temp_coord.y = value;
  }
else
  {
    fprintf(stderr, "ERROR: bad client value in Xg_get_temp_coord. Aborted\n");
    exit(1);
  }

XtFree(buf);
}

/**************************************/
void Xg_set_label_vals(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;
  int
    i;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;
XtDestroyWidget(w);

if ((graph->temp_coord.x < 0.) || (graph->temp_coord.x > 100.))
  {
    Xg_error_sys(XgERR_BAD_COORDS, graph);
    return;
  }
if ((graph->temp_coord.y < 0.) || (graph->temp_coord.y > 100.))
  {
    Xg_error_sys(XgERR_BAD_COORDS, graph);
    return;
  }

switch ((int) wid_addr->client)
  {
  case XgXaxisLabel:
    graph->x_axis_desc.coord.x = graph->temp_coord.x;
    graph->x_axis_desc.coord.y = graph->temp_coord.y;
    strcpy(graph->x_axis_desc.text, graph->temp_text);
    if (strcmp(graph->x_axis_desc.font_info.name, graph->temp_font))
      {   /** new font **/
	XtFree(graph->x_axis_desc.font_info.font_struct);
	graph->x_axis_desc.font_info.font_struct
	  = XLoadQueryFont(graph->display, graph->temp_font);
	if (graph->x_axis_desc.font_info.font_struct == NULL)
	  {
	    graph->x_axis_desc.font_info.font_struct
	      = XLoadQueryFont(graph->display,
			       graph->x_axis_desc.font_info.name);
	    Xg_error_sys(XgERR_NO_FONT, graph);
	  }
	else
	  strcpy(graph->x_axis_desc.font_info.name, graph->temp_font);
      }
    break;
  case XgYLaxisLabel:
    graph->y_axis_desc.coord.x = graph->temp_coord.x;
    graph->y_axis_desc.coord.y = graph->temp_coord.y;
    strcpy(graph->y_axis_desc.text, graph->temp_text);
    if (strcmp(graph->y_axis_desc.font_info.name, graph->temp_font))
      {   /** new font **/
	XtFree(graph->y_axis_desc.font_info.font_struct);
	graph->y_axis_desc.font_info.font_struct
	  = XLoadQueryFont(graph->display, graph->temp_font);
	if (graph->y_axis_desc.font_info.font_struct == NULL)
	  {
	    graph->y_axis_desc.font_info.font_struct
	      = XLoadQueryFont(graph->display,
			       graph->y_axis_desc.font_info.name);
	    Xg_error_sys(XgERR_NO_FONT, graph);
	  }
	else
	  strcpy(graph->y_axis_desc.font_info.name, graph->temp_font);
      }
    break;
  case XgYRaxisLabel:
    graph->y_rt_axis_desc.coord.x = graph->temp_coord.x;
    graph->y_rt_axis_desc.coord.y = graph->temp_coord.y;
    strcpy(graph->y_rt_axis_desc.text, graph->temp_text);
    if (strcmp(graph->y_rt_axis_desc.font_info.name, graph->temp_font))
      {   /** new font **/
	XtFree(graph->y_rt_axis_desc.font_info.font_struct);
	graph->y_rt_axis_desc.font_info.font_struct
	  = XLoadQueryFont(graph->display, graph->temp_font);
	if (graph->y_rt_axis_desc.font_info.font_struct == NULL)
	  {
	    graph->y_rt_axis_desc.font_info.font_struct
	      = XLoadQueryFont(graph->display,
			       graph->y_rt_axis_desc.font_info.name);
	    Xg_error_sys(XgERR_NO_FONT, graph);
	  }
	else
	  strcpy(graph->y_rt_axis_desc.font_info.name, graph->temp_font);
      }
    break;
  case XgTitle:
    graph->title_desc.coord.x = graph->temp_coord.x;
    graph->title_desc.coord.y = graph->temp_coord.y;
    strcpy(graph->title_desc.text, graph->temp_text);
    if (strcmp(graph->title_desc.font_info.name, graph->temp_font))
      {   /** new font **/
	XtFree(graph->title_desc.font_info.font_struct);
	graph->title_desc.font_info.font_struct
	  = XLoadQueryFont(graph->display, graph->temp_font);
	if (graph->title_desc.font_info.font_struct == NULL)
	  {
	    graph->title_desc.font_info.font_struct
	      = XLoadQueryFont(graph->display,
			       graph->title_desc.font_info.name);
	    Xg_error_sys(XgERR_NO_FONT, graph);
	  }
	else
	  strcpy(graph->title_desc.font_info.name, graph->temp_font);
      }
    break;
  case XgSubtitle:
    graph->subtitle_desc.coord.x = graph->temp_coord.x;
    graph->subtitle_desc.coord.y = graph->temp_coord.y;
    strcpy(graph->subtitle_desc.text, graph->temp_text);
    if (strcmp(graph->subtitle_desc.font_info.name, graph->temp_font))
      {   /** new font **/
	XtFree(graph->subtitle_desc.font_info.font_struct);
	graph->subtitle_desc.font_info.font_struct
	  = XLoadQueryFont(graph->display, graph->temp_font);
	if (graph->subtitle_desc.font_info.font_struct == NULL)
	  {
	    graph->subtitle_desc.font_info.font_struct
	      = XLoadQueryFont(graph->display,
			       graph->subtitle_desc.font_info.name);
	    Xg_error_sys(XgERR_NO_FONT, graph);
	  }
	else
	  strcpy(graph->subtitle_desc.font_info.name, graph->temp_font);
      }
    break;
  case XgLegend:
    graph->legend.description.coord.x = graph->temp_coord.x;
    graph->legend.description.coord.y = graph->temp_coord.y;
    strcpy(graph->legend.description.text, graph->temp_text);
    if (strcmp(graph->legend.description.font_info.name, graph->temp_font))
      {   /** new font **/
	XtFree(graph->legend.description.font_info.font_struct);
	graph->legend.description.font_info.font_struct
	  = XLoadQueryFont(graph->display, graph->temp_font);
	if (graph->legend.description.font_info.font_struct == NULL)
	  {
	    graph->legend.description.font_info.font_struct
	      = XLoadQueryFont(graph->display,
			       graph->legend.description.font_info.name);
	    Xg_error_sys(XgERR_NO_FONT, graph);
	  }
	else
	  strcpy(graph->legend.description.font_info.name, graph->temp_font);
      }
    break;
  default:
    fprintf(stderr, "ERROR: Bad Label type in Xg_set_label_vals\n");
    break;
  }

}

/**************************************/
void Xg_titles_defaults(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;
  int
    i;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;
XtDestroyWidget(w);

switch ((int) wid_addr->client)
  {
  case XgXaxisLabel:
    graph->x_axis_desc.coord.x = XgDefXtextXCoord;
    graph->x_axis_desc.coord.y = XgDefXtextYCoord;
    XtFree(graph->x_axis_desc.font_info.font_struct);
    graph->x_axis_desc.font_info.font_struct
      = XLoadQueryFont(graph->display, XgDefFont);
    strcpy(graph->x_axis_desc.font_info.name, XgDefFont);
    break;
  case XgYLaxisLabel:
    graph->y_axis_desc.coord.x = XgDefYtextXCoord;
    graph->y_axis_desc.coord.y = XgDefYtextYCoord;
    XtFree(graph->y_axis_desc.font_info.font_struct);
    graph->y_axis_desc.font_info.font_struct
      = XLoadQueryFont(graph->display, XgDefFont);
    strcpy(graph->y_axis_desc.font_info.name, XgDefFont);
    break;
  case XgYRaxisLabel:
    graph->y_rt_axis_desc.coord.x = XgDefYrtTextXCoord;
    graph->y_rt_axis_desc.coord.y = XgDefYrtTextYCoord;
    XtFree(graph->y_rt_axis_desc.font_info.font_struct);
    graph->y_rt_axis_desc.font_info.font_struct
      = XLoadQueryFont(graph->display, XgDefFont);
    strcpy(graph->y_rt_axis_desc.font_info.name, XgDefFont);
    break;
  case XgTitle:
    graph->title_desc.coord.x = XgDefTitletextXCoord;
    graph->title_desc.coord.y = XgDefTitletextYCoord;
    XtFree(graph->title_desc.font_info.font_struct);
    graph->title_desc.font_info.font_struct
      = XLoadQueryFont(graph->display, XgDefFont);
    strcpy(graph->title_desc.font_info.name, XgDefFont);
    break;
  case XgSubtitle:
    graph->subtitle_desc.coord.x = XgDefSubtextXCoord;
    graph->subtitle_desc.coord.y = XgDefSubtextYCoord;
    XtFree(graph->subtitle_desc.font_info.font_struct);
    graph->subtitle_desc.font_info.font_struct
      = XLoadQueryFont(graph->display, XgDefFont);
    strcpy(graph->subtitle_desc.font_info.name, XgDefFont);
    break;
  case XgLegend:
    graph->legend.description.coord.x = XgDefLegendXCoord;
    graph->legend.description.coord.y = XgDefLegendYCoord;
    XtFree(graph->legend.description.font_info.font_struct);
    graph->legend.description.font_info.font_struct
      = XLoadQueryFont(graph->display, XgDefFont);
    strcpy(graph->legend.description.font_info.name, XgDefFont);
    graph->legend.use_legend = XgDefLegendOnOff;
    break;
  default:
    break;
  }

}

/************************/
void Xg_toggle_legend(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[5];
  XgGraphInfo
    *graph;
  XmString
    label;
  XmToggleButtonCallbackStruct
    *toggle_info;

toggle_info = (XmToggleButtonCallbackStruct *) call_data;
graph = (XgGraphInfo *) client_data;

if (toggle_info->set == True)
  {
    graph->legend.use_legend = XgLegendOn;
    label = XmStringCreateLtoR("Legend ON/off", XgCharset);
  }
else if (toggle_info->set == False)
  {
    graph->legend.use_legend = XgLegendOff;
    label = XmStringCreateLtoR("Legend on/OFF", XgCharset);
  }
else
  fprintf(stderr, "ERROR in toggle_info->set value\n");

nargs = 0;
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(w, args, nargs);
XtFree(label);

}
