/**********************************************************************
*
*                              slc3d.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.2   5/92
*
*  This program reads an image volume and takes an X-Z row slice (default)
*   or a Y-Z column slice by specifying "-c" on the command line
*  Program for reading standard ETS data files and displaying header
*    info. Also shows pixel values if desired.
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xg3d.h"
#include "etsheadr.h"
#define SLCROW 1
#define SLCCOL 2

main(argc, argv)
     int
       argc;
     char
       **argv;
{
  struct fileinfo
    datafile;
  char
    buf[80],
    *header_char;
  int2
    tmp,
    *tmp_array,
    *pix_array;
  int
    i,
    j,
    k,
    index,
    nrows,
    ncols,
    nframes,
    nx,
    ny,
    slcrorc,
    slice_num;
  double
    scale_fac;
  Widget
    graph;
  Display
    *display;
  XtAppContext
    theAppContext;
  Xg3dGraphInfo
    graphinfo;
slcrorc = SLCROW;
header_char = (char *) &datafile.file_header;
datafile.filename[0] = '\0';  /* make sure we don't open garbage */
for (i=1; i<argc; ++i) /* find file name if given */
  {
    /** if this is - option, skip it & possibly next arg **/
    if ((!strcmp(argv[i], "-h")) || (!strcmp(argv[i], "-c"))
	|| (!strcmp(argv[i], "-t")));
    else if (!strncmp(argv[i], "-", 1))
      ++i;
    else /** assume this is file name **/
      strcpy(datafile.filename, argv[i]);
  }
slice_num = 1;
scale_fac = 0;
for (i=1; i<argc; ++i)
  {
    if (!strcmp(argv[i], "-f"))
      {
	sscanf(argv[i+1], "%d", &slice_num);
      }
    if (!strcmp(argv[i], "-s"))
      {
	sscanf(argv[i+1], "%le", &scale_fac);
	printf("slc3d: Selected scale factor %lg\n", scale_fac);
      }
    if (!strcmp(argv[i], "-c"))
      {
	slcrorc = SLCCOL;
      }
    if (!strncmp(argv[i], "-h", 2)) /** check for help **/
      {
	slc3dhlp();
	exit(0);
      }
  }
fileopen(&datafile);
/*showhead(&datafile);*/
nrows = datafile.file_header.col_length;
ncols = datafile.file_header.row_length;
nframes = datafile.file_header.frames_good;
nx = nframes;
if (slcrorc == SLCCOL)
  ny = nrows;
else
  ny = ncols;
if ((slice_num < 1) || (slice_num > ny))
  slice_num = 1;   /* he blew it! */
if (slcrorc == SLCCOL)
  printf("slc3d: Selected column number %d\n", slice_num);
else
  printf("slc3d: Selected row number %d\n", slice_num);
pix_array = (int2 *) calloc(nx*ny, sizeof(int2));
if (pix_array == NULL)
  {
    fprintf(stderr, "Unable to create pix_array. Aborted\n");
    exit(0);
  }
/****************** get data *******************/
for (k=0; k<nframes; ++k)
  {
    printf("reading frame %d      \r", k+1);
    fflush(stdout);
    tmp_array = readframe(&datafile, 0);
    if (slcrorc == SLCCOL)
      for (i=0; i<ny; ++i)
	*(pix_array + i*nx + k) = *(tmp_array + i*ncols + slice_num);
    else
      for (i=0; i<ncols; ++i)
	*(pix_array + i*nx + k) = *(tmp_array + slice_num*ncols + i);
    free(tmp_array);
  }
printf("\n");
fileclose(datafile.dat_file);
for (i=1; i<argc; ++i)      /* transpose it */
  if (!strcmp(argv[i], "-t"))
    {
      printf("Rotating %d x %d array", nx, ny);
      tmp_array = (int2 *) calloc(nx*ny, sizeof(int2));
      if (tmp_array == NULL)
	{
	  fprintf(stderr, "Couldn't allocate rotation array. Aborted.\n");
	  exit(0);
	}
      for (i=0; i<ny; ++i)
	for (j=0; j<nx; ++j)
	    *(tmp_array + j*ny + i) = *(pix_array + i*nx + j);
      free(pix_array);
      pix_array = tmp_array;
      tmp = nx;
      nx = ny;
      ny = tmp;
      printf(" to %d x %d array\n", nx, ny);
    }
/**********************************/
Xg3dInitializeStruct(&graphinfo);
graphinfo.num_sets = 1;
graphinfo.dataset[0].darray = pix_array;
graphinfo.dataset[0].nx = nx;
graphinfo.dataset[0].ny = ny;
strcpy(graphinfo.dataset[0].filename, datafile.filename);
strcpy(graphinfo.x_axis_text, datafile.filename);
strcpy(graphinfo.y_axis_text, "Intensity");
strcpy(graphinfo.title_text, "Surface Plotter");
if (slcrorc == SLCCOL)
  sprintf(buf, "Image Data: Column %d", slice_num);
else
  sprintf(buf, "Image Data: Row %d", slice_num);
strcpy(graphinfo.subtitle_text, buf);
if (scale_fac > 0.)
  graphinfo.scale_fac = scale_fac;
/********************************/
XtToolkitInitialize();
theAppContext = XtCreateApplicationContext();
display = XtOpenDisplay(theAppContext, NULL, argv[0], "Xg3dPlot", NULL, 0,
			(Cardinal *) &argc, argv);
if (display == NULL)
  {
    fprintf(stderr, "Unable to open requested display. Aborting\n");
    exit(0);
  }
graph = XtAppCreateShell("Imager", Xg3dClass,
			    applicationShellWidgetClass, display, NULL, 0);
graphinfo.app_shell = graph;
Xg3dCreateGraph(&graphinfo);
XtAppMainLoop(theAppContext);
/****************** finish up *********************/
free(pix_array);
}

/********************/
slc3dhlp()
{
fprintf(stderr, "slc3d command line help:\n");
fprintf(stderr, " usage: slc3d [options]\n");
fprintf(stderr, "      where options are one or more of the following:\n");
fprintf(stderr, "   any of the standard Xt command line options\n");
fprintf(stderr, "   [(a data file name)] indicating startup data to plot\n");
fprintf(stderr, "   [-f (row number)]  default is row 1\n");
fprintf(stderr, "   [-s (scale factor)] default is 0.1\n");
fprintf(stderr, "   [-c] slice a column instead of a row\n");
fprintf(stderr, "   [-t] transpose from frames x columns to columns x frames\n");
fprintf(stderr, "   Data is expected to be in 'ETS' standard format as\n");
fprintf(stderr, "   defined in the etsheadr.h file. Must be 3-D data.\n");
}
